#pragma once 
#include <string>
#include "trace.h"

typedef 
   std::wstring 
   string ;

MIDL_INTERFACE( "660408A1-4FC2-472f-9127-27F07A700348" )
ILink : IUnknown
{
   virtual string const& __stdcall GetImageName  () = 0 ; // file or folder name without containing folder name prefix 
   virtual string const& __stdcall GetFtpServer  () = 0 ; // ftp name in address:port format
   virtual string const& __stdcall GetFtpLink    () = 0 ; // full ftp path 
   virtual BOOL          __stdcall IsFolder      () = 0 ; 
   virtual long          __stdcall GetSize       () = 0 ; 
   virtual void          __stdcall GetTime       ( short& year, short& month, short& date, 
                                                   short& hour, short& minute ) = 0 ;
};
_COM_SMARTPTR_TYPEDEF( ILink, __uuidof( ILink )) ;

MIDL_INTERFACE( "D268E288-E7EF-45a4-8EA4-DD803F2CA473" ) 
ILinksIterator : IUnknown 
{
   virtual void      __stdcall Reset() = 0 ; // allows to iterate from the begining again 
   virtual ILinkPtr  __stdcall Next () = 0 ; // return NULL if ends 
}; 
_COM_SMARTPTR_TYPEDEF( ILinksIterator, __uuidof( ILinksIterator )) ;

MIDL_INTERFACE( "411CCEA6-BCFC-4703-A0AE-6C31DB4DB738" )
ILinksProvider : IUnknown 
{
   typedef string string ;

   // server name without "ftp:\\" prefix 
   virtual BOOL __stdcall GetFtpAccount( string const& ftpServer, LPWSTR user, LPWSTR pass, size_t& size ) = 0 ;

   // creates an iterator through the given folder, returns NULL if no such folder found  
   virtual ILinksIteratorPtr __stdcall CreateItemIterator( string const& folderName ) = 0 ;
   virtual ILinkPtr          __stdcall Search            ( string const& fileName   ) = 0 ;           
};
_COM_SMARTPTR_TYPEDEF( ILinksProvider, __uuidof( ILinksProvider )) ;

MIDL_INTERFACE( "B9818B2C-E975-43db-A510-7A11DA2A6494" )
IAuthentication : IUnknown
{
   enum USER_PRIVILEGE 
   {
      UP_NO_USER     = 0,
      UP_NEED_PASW   = 1, 
      UP_NO_PASW     = 2
   };

   virtual USER_PRIVILEGE __stdcall CheckUser   ( string const& user ) = 0 ;
   virtual BOOL           __stdcall CorrectPassw( string const& user, string const& passw ) = 0 ;
};
_COM_SMARTPTR_TYPEDEF( IAuthentication, __uuidof( IAuthentication )) ;

// main server central point 
MIDL_INTERFACE( "86856E0A-2F98-4897-B1B8-4C4DE3E3F803" ) 
IFTPServer : IUnknown
{
   virtual void __stdcall StopServer   ( BOOL force = FALSE ) = 0 ;
   virtual BOOL __stdcall StartServer  ( ILinksProvider* provider, ITracerManager* tracermng, 
                                         short port = 21, DWORD ip = 0x0100007f ) = 0  ;
};
_COM_SMARTPTR_TYPEDEF( IFTPServer, __uuidof ( IFTPServer )) ;

class __declspec( uuid( "6BA37B67-A1E6-4c48-A2F6-EAF6DAE93649" )) CFTPServer ;
