#ifndef __ATL_EXT_H
#define __ATL_EXT_H

template < class Interface, class Ptr >
inline void ptrcpy( Interface ** pp, Ptr p )
{
   *pp = static_cast<Interface *>(p) ; 
   if ( *pp ) 
      (*pp) -> AddRef () ; 
} 

template <class Event, class Base = Event>
    struct Listener : Base
{
    Listener() : m_dwCookie(0) {}

    bool Init(IUnknown *environment, Event *pThis)
    {
        _com_util::CheckError(
            AtlAdvise(m_Container = environment, pThis, __uuidof(Event), &m_dwCookie));
        // todo: report if advise fails
        return (m_dwCookie != 0);
    }

    void Init(IUnknown *environment, IUnknownPtr pThis)
    {
        _com_util::CheckError(
            AtlAdvise(m_Container = environment, pThis, __uuidof(Event), &m_dwCookie));        

        // todo: report if advise fails
    }

    void Deinit()
    {
        _com_util::CheckError(
            AtlUnadvise(m_Container, __uuidof(Event), m_dwCookie));

        m_dwCookie = 0;
        m_Container = 0;
    }

private:
    DWORD       m_dwCookie;
    IUnknownPtr m_Container;
};

//-- for non-COM listeners 
template< class Owner, class Event, class ComThreadModel = CComMultiThreadModel >
struct EventListenerImpl
  : public CComObjectRootEx< ComThreadModel >
  , public Listener< Event > 
{
  BEGIN_COM_MAP( EventListenerImpl )
    COM_INTERFACE_ENTRY( Event )
  END_COM_MAP  ()

  void Init ( IUnknown* environment, Owner* owner ) 
  {
    owner_ = owner ; 
    Listener< Event >::Init( environment, GetUnknown()) ;
  }

  void Deinit()
  {
    Listener< Event >::Deinit() ;
  }

protected:
  Owner* owner_ ;
};

template < class Owner, template < class > class EventListenerProxy >  
struct Subscriber
{
  Subscriber ( IUnknown* environment, Owner* owner )
  {
    if ( SUCCEEDED( ListenerProxyComObj::CreateInstance( &listener_ )))
    {
      listener_ -> AddRef() ;
      listener_ -> Init( environment, owner ) ;
    }
    else 
      listener_ = 0 ;
  }

  ~Subscriber ()
  {
    if ( listener_ )
    {
      listener_ -> Deinit () ;
      listener_ -> Release() ;
      listener_ = 0 ;
    }
  } 

private:
  typedef
    CComObject< EventListenerProxy < Owner > >
    ListenerProxyComObj ;

private:
  ListenerProxyComObj* listener_ ;
} ; 

// NOTE : make proxy in this way:

// template < class Owner >  
// struct MyEventListenerProxy
//   : public EventListenerImpl< Owner, IMyEvent >
// {
//   // IMyEvent implementation 
//   void __stdcall OnMyMethod () 
//   { 
//     if ( owner_ ) 
//       owner_->OnMyMethod() ;
//   }
// };

// Owner sample code:

// class Owner
// {
// public:
//   void Init( IUnknown *environment, ... )
//   {
//     subscriber_.reset( new Subscriber< Owner, MyEventListenerProxy >( environment, this ));
//     ...
//   }
// 
//   void Deinit()
//   {
//     ...
//     subscriber_.reset();
//   }
// 
//   void OnMyMethod()
//   { 
//     ...
//   }
//
// private:
//   boost::scoped_ptr< Subscriber< Owner, MyEventListenerProxy > > subscriber_;
// };

// class Owner
// {
// public:
//   Owner ( IUnknown *environment )
//       : subscirber_ (environment, this)
//   {
//     ...
//   }
// 
//   void OnMyMethod()
//   { 
//     ...
//   }
//
// private:
//   Subscriber< Owner, MyEventListenerProxy > subscriber_;
// };

//--


template <class T1>
class CComSingleInstanceCreator : protected CComCreator<T1>
{
public:
  static HRESULT WINAPI CreateInstance(void* pv, REFIID riid, LPVOID* ppv)
     {
    _ASSERTE(*ppv == NULL);
          if ( T1::Instance () )
               return T1::Instance () -> _InternalQueryInterface(riid, ppv) ;
          else return CComCreator<T1> :: CreateInstance( pv, riid, ppv) ;
     }
} ;

template <class T1>
class CSingleWrapper : public T1
{
public :
     static T1 *& Instance () { static T1 * instance = NULL ; return instance ; }
     CSingleWrapper () : T1 ()
     {
         Instance () = this ;
     }
    ~CSingleWrapper ()
     {
         Instance () = NULL ;
     }
} ;

#define DECLARE_SINGLE_INSTANCE(x) public:\
  typedef CComCreator2< CComSingleInstanceCreator< CComObject< CSingleWrapper < x > > >, CComFailCreator<CLASS_E_NOAGGREGATION> > _CreatorClass; 

#define DECLARE_PROGID(x)\
  static LPCOLESTR WINAPI GetObjectProgId()\
  {\
    return L##x;\
  }

#define DECLARE_NO_PROGID()\
  static LPCOLESTR WINAPI GetObjectProgId()\
  {\
    return L"???";\
  }


#define DECLARE_DESCRIPTION(x)\
  static LPCOLESTR WINAPI GetObjectDescription2()\
  {\
    return L##x;\
  }


#define DECLARE_REGISTRY_RESOURCEID_EX(x)\
  static HRESULT WINAPI UpdateRegistry(BOOL bRegister)\
  {\
    _ATL_REGMAP_ENTRY rm[] =\
    {\
      { L"CLSID",  0 },\
      { L"PROGID", _CoClass::GetObjectProgId() },\
      { L"DESCRIPTION", _CoClass::GetObjectDescription2() },\
      {0,0}\
    };\
    StringFromCLSID ( GetObjectCLSID(), (LPOLESTR*)&rm[0].szData );\
    HRESULT hr = _pAtlModule -> UpdateRegistryFromResource(x, bRegister, rm);\
    CoTaskMemFree ( (void*)rm[0].szData );\
    return hr;\
  }

#define DECLARE_REGISTRY_RESOURCEID_STRING(x,s)\
   static HRESULT WINAPI UpdateRegistry(BOOL bRegister)\
   {\
      _ATL_REGMAP_ENTRY rm[] =\
      {\
         { L"CLSID",  0 },\
         { L"PROGID", _CoClass::GetObjectProgId() },\
         { L"DESCRIPTION", _CoClass::GetObjectDescription2() },\
         {0,0}\
      };\
      StringFromCLSID ( GetObjectCLSID(), (LPOLESTR*)&rm[0].szData );\
      HRESULT hr = _pAtlModule -> UpdateRegistryFromResource(x, bRegister, rm);\
      CoTaskMemFree ( (void*)rm[0].szData );\
      if ( FAILED(hr) ) return hr ;\
      CRegObject ro ;\
      for ( _ATL_REGMAP_ENTRY * pme = rm; pme->szKey != NULL; ++pme )\
         ro.AddReplacement(pme->szKey, pme->szData);\
      hr = bRegister ? ro.StringRegister ( s ) : ro.StringUnregister ( s ); \
      return hr;\
   }


//    StringFromCLSID ( __uuidof(_ComMapClass), (LPOLESTR*)&rm[0].szData );

#define COM_INTERFACE_ENTRY_AGGREGATE_BLIND_EX(ppunk)\
  {NULL,\
  (DWORD)offsetof(_ComMapClass, ppunk),\
  _DelegateEx},
 
static HRESULT WINAPI _DelegateEx(void* pv, REFIID iid, void** ppvObject, DWORD dw)
{
  HRESULT hRes = E_NOINTERFACE;
  CComPtr<IUnknown> * pvec = (CComPtr<IUnknown> *)((DWORD)pv + dw);
  
  CComPtr<IUnknown> p;
  
  int i = 0 ;
  while ( (p = pvec[i++]) != NULL )
  {
    hRes = p -> QueryInterface ( iid, ppvObject );
    if ( hRes == S_OK ) break;
  }
 
  return hRes;
}

template < class T >
IUnknown * CreateInstance ()
{
  CComObject<T> * t ; 
  if ( CComObject<T> :: CreateInstance ( &t ) == S_OK ) 
       return t -> GetUnknown () ; 
  else return NULL ; 
}  

#endif
