#pragma once 
#include <stdio.h>
#include <stdarg.h>

MIDL_INTERFACE( "8DE0520E-9545-4947-9ADD-09E0FAEF66EF" )
ITracer : IUnknown
{
   virtual void __stdcall Trace( LPCWSTR msg ) = 0 ;
};
_COM_SMARTPTR_TYPEDEF( ITracer, __uuidof( ITracer )) ;

// tracing messages
MIDL_INTERFACE( "4CA77ACA-5A6D-4482-9E36-6462D70947DC" )
ITracerManager : IUnknown
{
   virtual ITracerPtr __stdcall CreateTracer( LPCWSTR name ) = 0 ;
};
_COM_SMARTPTR_TYPEDEF( ITracerManager, __uuidof( ITracerManager )) ;

// static Tracer, for common trace 
struct StaticTracer 
{
   static void Init  ( ITracer* tracer )
   {
      tracer_ = tracer ;
   }

   static void Deinit()
   {
      tracer_ = NULL ;
   }

   static void Trace ( LPCWSTR msg )
   {
      if ( tracer_ )
         tracer_ -> Trace( msg ) ;
   }

private:
   static ITracer* tracer_ ; 
};

__declspec( selectany ) ITracer* StaticTracer::tracer_ = 0 ;

inline void __TRACE ( LPCWSTR format, /*arguments*/... ) 
{
   wchar_t buf [0x400] ;
   va_list arglist ;
   va_start( arglist, format ) ;

   vswprintf(( LPWSTR ) buf, format, arglist ) ;
   StaticTracer::Trace( buf ) ;
}

struct TracerImpl 
{
   TracerImpl( ITracer* tracer = NULL )
      : tracer_( tracer )
   {
   }

   TracerImpl( ITracerManager* mng, LPCWSTR streamName )
   {
      Init( mng, streamName ) ;
   }

   BOOL Init( ITracer* tracer )
   {
      tracer_ = tracer ;
      return tracer_ ;
   }

   BOOL Init( ITracerManager* mng, LPCWSTR streamName )
   {
      if ( streamName != NULL && mng != NULL )
         tracer_ = mng->CreateTracer( streamName ) ;

      return tracer_ != NULL ;
   }
   
   void Deinit()
   {
      tracer_ = NULL ;
   }

   void Trace( LPCWSTR format, ... )                             
   {                                                                   
      wchar_t buf [0x400] ;                                            

      va_list arglist ;                                                
      va_start( arglist, format )  ;                                   

      vswprintf( LPWSTR ( buf ), format, arglist ) ;                   

      SYSTEMTIME st ;
      GetSystemTime( &st ) ;

      wchar_t log [0x400] ;
      swprintf( log, L"[%d:%d:%d.%d] %s%s", st.wHour, st.wMinute, st.wSecond, st.wMilliseconds, GetPrefix(), buf ) ;

      if ( tracer_ )
         tracer_->Trace( log ) ;                                           
   }

   ITracerPtr GetTracer() const 
   {
      return tracer_ ;
   }

protected:
   virtual LPCWSTR GetPrefix() const { return L"" ; }

private:
   ITracerPtr tracer_ ;       
};



