#include "stdafx.h"
#include "CentralPoint.h"

//////////////////////////////////////////////////////////////////////////
//
// Listener
//

Listener::Listener( CCentralPoint& cp ) 
   : TListener ( AF_INET, SOCK_STREAM, IPPROTO_TCP )
   , cp_       ( cp )
{
}

void Listener::Accepted( SOCKET sock ) 
{ 
   cp_.Accepted( sock ) ; 
}

//////////////////////////////////////////////////////////////////////////
//
// Central Point 
//

CCentralPoint :: CCentralPoint() 
   : automata_    ( *this )
   , lastId_      ( 0 )
   
   , listenAddr_  ( 21, 0x0100007f )
   , newUserSock_ ( INVALID_SOCKET )
{
   automata_.Init( 0 ) ;
}

CCentralPoint ::~CCentralPoint()
{
}

void CCentralPoint :: StopServer( BOOL forceStop )
{
   automata_.ProcessEvent( forceStop 
                  ? central_point::E_3_STOPFORCE 
                  : central_point::E_4_STOPNONFORCE ) ;
}

BOOL CCentralPoint :: StartServer( ILinksProvider* provider, ITracerManager* tracermng, short port, DWORD ip )
{
   TracerImpl  ::Init( tracermng, L"Server.log" ) ;
   StaticTracer::Init( GetTracer()) ;

   automata_.SetTracer( GetTracer()) ;
   automata_.DumpState() ;

   linksProvider_ = provider  ;
   tracerMng_     = tracermng ;
   listenAddr_    = TINETAddr ( port, ip ) ;

   automata_.ProcessEvent( central_point::E_1_START ) ;

   return TRUE ;
}

void CCentralPoint::UserDropped( long Id ) 
{
   UsersMap::iterator it = users_.find( Id ) ;
   if( it != users_.end())
   {
      droppedUsers_.push_back( it->first ) ;
      automata_.ProcessEvent( central_point::E_6_USERDISCONNECTED ) ;
   }
}

void CCentralPoint :: Accepted( SOCKET sock )
{
   newUserSock_ = sock ;
   automata_.ProcessEvent( central_point::E_2_NEWUSER ) ;
}

//////////////////////////////////////////////////////////////////////////
// automaton routine 

BOOL CCentralPoint :: x1() const { return linksProvider_ != NULL ; } // has links provider
BOOL CCentralPoint :: x2() const { return listening_ ; }             // listening 
BOOL CCentralPoint :: x3() const { return !users_.empty() ; }        // has users 

void CCentralPoint :: z1() // add user  
{  
   users_[ lastId_ ].reset( new UserSite( newUserSock_, *this, linksProvider_, lastId_ , tracerMng_ )) ;
   
   ++lastId_ ;
   ATLASSERT( lastId_ < LONG_MAX ) ;
   newUserSock_ = INVALID_SOCKET ;
} 

void CCentralPoint :: z2()  // delete all users && stop listening 
{
   users_.clear() ;
   listener_.reset() ;
   listening_ = FALSE ;

   TracerImpl::Deinit() ;

   tracerMng_     = NULL ;
   linksProvider_ = NULL ;
}

void CCentralPoint :: z3()  // say users to stop 
{
   for( UsersMap::iterator it = users_.begin(); it != users_.end(); ++it )
      it->second->Stop() ;
}

void CCentralPoint :: z4()  // start listening 
{
   listener_.reset() ;

   listener_.reset( new Listener( *this )) ;
   listening_ = listener_->ListenAs( listenAddr_ ) ;
}

void CCentralPoint :: z5() // remove user 
{
   for( size_t i = 0, size = droppedUsers_.size(); i < size; ++i )
   {
      UsersMap::iterator it = users_.find( droppedUsers_[ i ] ) ;
      if( it != users_.end())
         users_.erase( it ) ;
   }

   droppedUsers_.clear() ; 

   if( users_.empty())
      automata_.ProcessEvent( central_point::E_5_LASTDROPPED ) ;
}
