#pragma once 
#include "UserSite.h"
#include "Interfaces.h"
#include "resource.h"
#include "Trace.h"

#include "CPAutomaton.h"

// forward declaration 
class CCentralPoint ;

struct Listener
   : public TListener 
{
   Listener( CCentralPoint& cp ) ; 

// TListener
private:
   void Accepted( SOCKET sock ) ; 

private:
   CCentralPoint& cp_ ;
};

class CCentralPoint 
   : public CComObjectRootEx < CComMultiThreadModel >  
   , public CComCoClass < CCentralPoint, &__uuidof( CFTPServer ) > 
   , public IFTPServer
   , public TracerImpl
{
public:
// COM routine    
   DECLARE_REGISTRY( CCentralPoint, "mp3ftp.FTPServer.CentralPoint", "FTPServer", IDS_PROJECT, THREADFLAGS_APARTMENT ) ;

   BEGIN_COM_MAP( CCentralPoint )
      COM_INTERFACE_ENTRY( IFTPServer )
   END_COM_MAP  ()

public:
   CCentralPoint() ;
  ~CCentralPoint() ;

// IFTPServer
public:
   void __stdcall StopServer  ( BOOL force ) ;
   BOOL __stdcall StartServer ( ILinksProvider*, ITracerManager* tracermng, short port, DWORD ip )  ;
 
//////////////////////////////////////////////////////////////////////////
// Automaton routine 
//
public:
   BOOL x1() const ; // has links provider
   BOOL x2() const ; // could listen 
   BOOL x3() const ; // has users 

   void z1() ; // add user 
   void z2() ; // delete all users && stop listening 
   void z3() ; // say users to stop 
   void z4() ; // start listening 
   void z5() ; // remove user 
//
//////////////////////////////////////////////////////////////////////////

public:
   void UserDropped( long Id ) ;

// for listener 
public:
   void Accepted( SOCKET sock ) ;

private:
   LPCWSTR GetPrefix() const { return L"CentralPoint Object: " ; }

private:
   // Interconnection with user 
   typedef  UserSite::PUserSite           PUserSite      ;
   // users are identified by long 
   typedef  std::map< long, PUserSite >   UsersMap       ; 
   typedef  std::vector< long >           DroppedUsers   ;

   typedef  boost::scoped_ptr< Listener > ListenerRef ;

private:
   central_point::A1_Main automata_ ;

private:
   UsersMap       users_         ;
   long           lastId_        ;
   DroppedUsers   droppedUsers_  ;

private:
   ILinksProviderPtr linksProvider_ ;
   ITracerManagerPtr tracerMng_     ;

// saving parameters 
private:
   TINETAddr      listenAddr_    ;
   BOOL           listening_     ;
   SOCKET         newUserSock_   ;
   PUserSite      droppedUser_   ;

private: 
   ListenerRef    listener_      ;
};

OBJECT_ENTRY_AUTO( __uuidof( CFTPServer ), CCentralPoint ) ;