#pragma once 

struct Command ;

struct CommandParser
{
   enum CMD
   {
      NONE  ,

      USER  ,
      PASS  ,

      CWD   ,
      PWD   ,
      QUIT  ,
      REIN  ,
      PORT  ,
      TYPE  ,
      STRU  ,
      MODE  ,
      ABOR  ,
      NOOP  ,

      LIST  ,
      RETR  
   } ;

   struct Command
   {
      CommandParser::CMD cmd ; 
      std::string        arg ;
   };

   static void Init()
   {
      if( cmdTable_.empty())
      {
         cmdTable_[ "USER" ] = USER ;
         cmdTable_[ "PASS" ] = PASS ;
                              
         cmdTable_[ "CWD"  ] = CWD  ;
         cmdTable_[ "PWD"  ] = PWD  ;
         cmdTable_[ "QUIT" ] = QUIT ;
         cmdTable_[ "PORT" ] = PORT ; 
         cmdTable_[ "TYPE" ] = TYPE ;
         cmdTable_[ "STRU" ] = STRU ; 
         cmdTable_[ "MODE" ] = MODE ;
         cmdTable_[ "ABOR" ] = ABOR ;
         cmdTable_[ "NOOP" ] = NOOP ; 
                              
         cmdTable_[ "LIST" ] = LIST ;
         cmdTable_[ "RETR" ] = RETR ;
      }
   }

   static std::wstring Format( size_t size, LPCSTR line )
   {
      USES_CONVERSION ;
      return string( A2W( line ), size ) ;
   }

   static BOOL Parse( size_t size, LPCSTR line, Command& cmd )
      {
      char buf[ 0x100 ] ;
      memset ( buf, 0, 0x100 ) ;
      strncpy( buf, line, size ) ;

      LPSTR p = strchr( buf, ' ' ) ; 
      if( p )
      {
         *p = 0 ;
         ++ p ;
      }

      LPSTR cmdName = buf ;
      
      // let's take only last maximum 4 bytes for command forming 
      size_t len = strlen( cmdName ) ;
      if ( len > 4 )
         cmdName += len - 4 ;

      CommandTable::const_iterator it = cmdTable_.find( std::string( cmdName )) ;
      if ( it != cmdTable_.end())
      {
         cmd.cmd = it->second ;
         cmd.arg = p ? p : "" ;
      
         return TRUE ;
      }

      cmd.cmd = NONE ;
      cmd.arg = ""   ;
      
      return FALSE ;
   }

private:
   typedef std::map< std::string, CMD > CommandTable ;  
   static CommandTable cmdTable_ ;
};

__declspec( selectany ) CommandParser::CommandTable CommandParser::cmdTable_ ;
