//////////////////////////////////////////////////////////////////////////
//
// RSAutomaton.cpp 
// Provides A5_RemoteServer definition 
//
//////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "RemoteServer.h"
#include "RSAutomaton.h"

namespace remote_server
{

A5_RemoteServer::A5_RemoteServer( RemoteServer& rs ) 
   : Base( main_automaton::S_1_CONNECT )
   , object_( rs )
{
}

void A5_RemoteServer::CalcState( OptState& state ) 
{
   using namespace main_automaton ;
   
   switch( GetState())
   {
   case S_1_CONNECT : 

      if( e( 6 ))
      {
         object_.z2() ;
         state.reset( S_8_CLOSE ) ;
         Trace( L"Transition 1" ) ;
      }
      else if( e( 7 ))
      {
         state.reset( S_2_WAIT4GREETINGS ) ;
         Trace( L"Transition 2" ) ;
      }

      break ;
   case S_2_WAIT4GREETINGS:

      if( e( 2 ))
      {
         state.reset( S_3_PROC_USER ) ;
         Trace( L"Transition 1" ) ;
      }
      else if( e( 6 ))
      {
         object_.z2 () ;
         state.reset( S_8_CLOSE ) ;
         Trace( L"Transition 2" ) ;
      }

      break;
   case S_3_PROC_USER : 
   
      if( e( 2 ) && !object_.x1())
      {
         object_.z11() ;
         state.reset( S_5_READY4RETR ) ;
         Trace( L"Transition 1" ) ;
      }
      else if( e( 3 )) // need password 
      {
         state.reset( S_4_PROC_PASS ) ;
         Trace( L"Transition 2" ) ;
      }
      else if( e( 1 ) || e ( 4 ) || e( 5 ) || e ( 6 ))
      {
         object_.z2() ;
         state.reset( S_8_CLOSE ) ;
         Trace( L"Transition 3" ) ;
      }
      else if( e( 2 ) && object_.x1())
      {
         object_.z15() ;
         object_.z17() ;
         state.reset( S_5_READY4RETR ) ;
         Trace( L"Transition 4" ) ;
      }

      break ;
   case S_4_PROC_PASS :

      if( e( 2 ) && !object_.x1())
      {
         object_.z11() ;
         state.reset( S_5_READY4RETR ) ;
         Trace( L"Transition 1" ) ;
      }
      else if ( e( 1 ) || e( 3 ) || e( 4 ) || e( 5 ) || e( 6 ))
      {
         object_.z2 () ;
         state.reset( S_8_CLOSE ) ;
         Trace( L"Transition 2" ) ;
      } 
      else if ( e( 2 ) && object_.x1())
      {
         object_.z15() ;
         object_.z17() ;
         state.reset( S_5_READY4RETR ) ;
         Trace( L"Transition 3" ) ;
      }

      break ;
   case S_5_READY4RETR : 

      if( e( 6 ))
      {
         state.reset( S_8_CLOSE ) ;
         Trace( L"Transition 1" ) ;
      }
      else if ( e( 8 ) && !object_.x1())
      {
         state.reset( S_6_SETPARAMS ) ;
         Trace( L"Transition 2" ) ;
      }

      break ;
   case S_6_SETPARAMS :

      if( e( 2 ))
      {
         object_.z9() ;
         state.reset( S_6_SETPARAMS ) ;
         Trace( L"Transition 1" ) ;
      }
      else if( e( 1 ) || e( 3 ) || e( 4 ) || e( 5 ) || e( 6 ))
      {
         object_.z16() ;
         state.reset( S_8_CLOSE ) ;
         Trace( L"Transition 2" ) ;
      }
      else if ( e( 9 ))
      {
         object_.z15() ;
         object_.z17() ;
         state.reset( S_5_READY4RETR ) ;
         Trace( L"Transition 3" ) ;
      }
      else if( object_.x4() && object_.x6())
      {
         object_.z7() ;
         state.reset( S_6_SETPARAMS ) ;
         Trace( L"Transition 4" ) ;
      }
      else if ( object_.x5() && object_.x6())
      {
         object_.z8() ;
         state.reset( S_6_SETPARAMS ) ;
         Trace( L"Transition 5" ) ;
      }
      else if ( object_.x2() && object_.x6())
      {
         object_.z5 () ;
         state.reset( S_6_SETPARAMS ) ;
         Trace( L"Transition 6" ) ;
      }
      else if( object_.x3() && object_.x6())
      {
         object_.z6 () ;
         state.reset( S_6_SETPARAMS ) ;
         Trace( L"Transition 7" ) ;
      }
      else if ( !object_.x2() && !object_.x3() && !object_.x4() && !object_.x5() && object_.x6())
      {
         object_.z10() ;
         state.reset( S_7_TRANSFERING ) ;
         Trace( L"Transition 8" ) ;
      }

      break ;
   case S_7_TRANSFERING :

      if( e( 6 ))
      {
         object_.z16() ;
         state.reset( S_8_CLOSE ) ;
         Trace( L"Transition 1" ) ;
      }
      else if ( e( 2 ) || e( 4 ) || e( 5 ))
      {
         object_.z13() ;
         state.reset( S_5_READY4RETR ) ;
         Trace( L"Transition 2" ) ;
      }
      else if ( e( 1 ))
      {
         object_.z12() ;
         state.reset( S_7_TRANSFERING ) ;
         Trace( L"Transition 3" ) ;
      }
      else if ( e( 9 )) 
      {
         object_.z18() ;
         object_.z15() ;
         object_.z17() ;
         state.reset( S_5_READY4RETR ) ;
         Trace( L"Transition 4" ) ;
      }

      break ;
   case S_8_CLOSE : 

      // no transitions 

      break ;
   }
}

void A5_RemoteServer::OnStateChanged() 
{
   using namespace main_automaton ;

   switch( GetState())
   {
   case S_1_CONNECT :

      object_.z1() ;

      break ;
   case S_3_PROC_USER :

      object_.z3() ;

      break ;
   case S_4_PROC_PASS :

      object_.z4() ;

      break ;
   case S_8_CLOSE :

      object_.z14() ;

      break ;
   }
}

} // namespace remote_server 
