#include "stdafx.h"
#include "RemoteServer.h"
#include "UserSite.h"
#include "Replies.h"

RemoteServer::RemoteServer( UserSite& us, ITracerManager* tracerMng, TINETAddr const& addr, 
                            LPCSTR user, LPCSTR pass, LPCSTR path )
   : auto_        ( *this  )
   , us_          ( us     )
   , user_        ( "USER ")
   , pass_        ( "PASS ")
   , path_        ( path   )
   , aborted_     ( FALSE  )
   , serverAddr_  ( addr   )
   , pendingSuccessEchos_( 0 )
{
   USES_CONVERSION ;

   user_ += std::string( user ) + "\r\n" ;
   pass_ += std::string( pass ) + "\r\n" ;

   wchar_t buf [ 0x80 ] ;
   char    adr [ 0x30 ] ;
   addr.GetText( 0x30, adr ) ;

   LPSTR p = strchr( adr, ':' ) ;
   p[ 0 ] = '-' ;

   swprintf( buf, L"remote_%d_%s.log", us_.ID(), A2W( adr )) ;

   TracerImpl::Init( tracerMng, buf ) ;
   auto_.Init( GetTracer()) ;
}

void RemoteServer::Download( LPCSTR path ) 
{
   path_ = path ;
   auto_.ProcessEvent( remote_server::E_8_RETRIEVE ) ;
}

// automaton routine 
BOOL RemoteServer::x1() const  // aborted
{
   return aborted_ ;
}
BOOL RemoteServer::x2() const  // need PORT
{
   return params_.addr != us_.TransfParams().addr ;
}
BOOL RemoteServer::x3() const  // need TYPE
{
   return params_.type != us_.TransfParams().type ;
}
BOOL RemoteServer::x4() const  // need STRU 
{
   return params_.stru != us_.TransfParams().stru ;
}
BOOL RemoteServer::x5() const  // need MODE 
{
   return params_.mode != us_.TransfParams().mode ;
}

BOOL RemoteServer::x6() const // all params sent 
{
   return pendingSuccessEchos_ == 0 ;
}

void RemoteServer::z1()  // connect to server
{
      ConnectTo( serverAddr_ ) ;
}
void RemoteServer::z2()  // process retrieving start procedure failure   
{
   us_.ProcStartRetr( rep550()) ;
   aborted_ = FALSE ;
}
void RemoteServer::z3()  // send USER
{
   SendCommand( user_.c_str()) ;
}
void RemoteServer::z4()  // send PASS
{
   SendCommand( pass_.c_str()) ;
}
void RemoteServer::z5()  // send PORT 
{
   char buf [ 0x40 ] ;
   TINETAddr const& addr = us_.TransfParams().addr ;
   u_short port = addr.port() ;
   in_addr ip ;
   ip.s_addr = addr.getip() ;

   sprintf( buf, "PORT %d,%d%,%d,%d,%d,%d\r\n", ip.S_un.S_un_b.s_b1, ip.S_un.S_un_b.s_b2, 
                                                ip.S_un.S_un_b.s_b3, ip.S_un.S_un_b.s_b4, 
                                                port / 256, port % 256 ) ;
   
   SendCommand( buf ) ;
   params_.addr = addr ;

   ++pendingSuccessEchos_ ;
}
void RemoteServer::z6()  // send TYPE 
{
   std::string cmd ( "TYPE " ) ;
   cmd += us_.TransfParams().type ;
   cmd += "\r\n" ;
   
   SendCommand( cmd.c_str()) ;
   params_.type = us_.TransfParams().type ;

   ++pendingSuccessEchos_ ;
}
void RemoteServer::z7()  // send STRU 
{
   std::string cmd ( "STRU " ) ;
   cmd += us_.TransfParams().stru ;
   cmd += "\r\n" ;

   SendCommand( cmd.c_str()) ;
   params_.stru = us_.TransfParams().stru ;

   ++pendingSuccessEchos_ ;
}
void RemoteServer::z8()  // send MODE 
{
   std::string cmd ( "MODE " ) ;
   cmd += us_.TransfParams().mode ;
   cmd += "\r\n" ;

   SendCommand( cmd.c_str()) ;
   params_.mode = us_.TransfParams().mode ;

   ++pendingSuccessEchos_ ;
}

void RemoteServer::z9()
{
   if ( pendingSuccessEchos_ > 0 )
      -- pendingSuccessEchos_ ;
}

void RemoteServer::z10() // send RETR  
{
   std::string cmd ( "RETR " ) ;
   cmd += path_  ;
   cmd += "\r\n" ;
   SendCommand( cmd.c_str()) ;
}
void RemoteServer::z11() // post e8 event 
{
   auto_.ProcessEvent( remote_server::E_8_RETRIEVE ) ;
}
void RemoteServer::z12() // forward retrieving start 
{
   us_.ProcStartRetr( lastReply_.c_str()) ;
}
void RemoteServer::z13() // forward retrieving stop 
{
   us_.ProcStopRetr( lastReply_.c_str()) ;
}
void RemoteServer::z14() // drop server connection 
{
   us_.DropRS( serverAddr_ ) ; 
}
void RemoteServer::z15() // notification about abnormal retrieving abort
{
   us_.Retranslate( rep426()) ;   
}
void RemoteServer::z16() // process retrieving procedure failure 
{
   us_.ProcStopRetr( rep451()) ;
}

void RemoteServer::z17() // notify about successful operation 
{
   us_.ProcStopRetr( rep226()) ;
   aborted_ = FALSE ;
}

void RemoteServer::z18() // send ABOR 
{
   SendCommand( "ABOR" ) ;
}

//////////////////////////////////////////////////////////////////////////
void RemoteServer::Abort()  // process abort ( from control connection of user site )
{
   aborted_ = TRUE ;
   auto_.ProcessEvent( remote_server::E_9_ABORT ) ;
}

//////////////////////////////////////////////////////////////////////////
// TelnetReceiver
void RemoteServer::FragmentReceived(int size, const void * data ) 
{
   USES_CONVERSION ;

   LPSTR buf = ( LPSTR ) _alloca( size + 1 ) ;
   strncpy( buf, (LPSTR) data, size ) ;
   buf[ size ] = 0 ;

   lastReply_  = buf ;
   Trace(( string( L"<-- " ) + FormatReply( lastReply_.c_str())).c_str()) ;
   
   lastReply_ += "\r\n" ;

   size_t  event = ParseReply( buf ) ;
   switch( event )
   {
   case 1:
      auto_.ProcessEvent( remote_server::E_1_REPLY ) ;
      break ;
   case 2:
      auto_.ProcessEvent( remote_server::E_2_REPLY ) ;
      break ;
   case 3:
      auto_.ProcessEvent( remote_server::E_3_REPLY ) ;
      break ;
   case 4:
      auto_.ProcessEvent( remote_server::E_4_REPLY ) ;
      break ;
   case 5:
      auto_.ProcessEvent( remote_server::E_5_REPLY ) ;
      break ;
   }
}

void RemoteServer::ConnectChanged() 
{
   if( connected )
      auto_.ProcessEvent( remote_server::E_7_CONNECTED ) ;
   else 
      auto_.ProcessEvent( remote_server::E_6_CONNECTIONERROR ) ; 
}

void RemoteServer::SendCommand( LPCSTR cmd ) 
{
   USES_CONVERSION ;
   Trace(( string( L"--> " ) + FormatReply( cmd )).c_str()) ;

   Send( strlen( cmd ), cmd ) ;
}
