//////////////////////////////////////////////////////////////////////////
//
// UserAutomata.cpp 
// Provides A2_MainAutomaton, A3_InitAutomaton and A4_DataTransfer definition 
//
//////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "UserSite.h"
#include "UserAutomata.h"

namespace user_site
{

//////////////////////////////////////////////////////////////////////////
//
// Init Automaton 
//

A3_InitAutomaton::A3_InitAutomaton( UserSite& us ) 
   : Base     ( init_automaton::S_1_WAIT4USER )
   , object_  ( us )
{
}

BOOL A3_InitAutomaton::IsInputEvent( Event event ) 
{
   return event >= E_1_CMD_USER && event <= E_2_CMD_PASS ;
}

void A3_InitAutomaton::CalcState( OptState& state ) 
{
   using namespace init_automaton ;

   switch( GetState())
   {
   case S_1_WAIT4USER:
      
      if( e(1))
      {
         state.reset( S_2_CHECKUSER ) ;
         Trace( L"Transition 1" ) ;
      }
      
      break ;
   case S_2_CHECKUSER:
      
      if( object_.x20() && object_.x21())
      {
         state.reset( S_3_WAIT4PASW ) ;
         Trace( L"Transition 1" ) ;
      }
      else if ( object_.x20() && !object_.x21() )
      {
         state.reset( S_5_SUCCEEDED ) ;
         Trace( L"Transition 2" ) ;
      }
      else 
      {
         state.reset( S_6_FAILED ) ;
         Trace( L"Transition 3" ) ;
      }

      break ;
   case S_3_WAIT4PASW:

      if( e( 2 ))
      {
         state.reset( S_4_CHECKPASW ) ;
         Trace( L"Transition 1" ) ;
      }

      break ;
   case S_4_CHECKPASW:

      if( object_.x22())
      {
         state.reset( S_5_SUCCEEDED ) ;
         Trace( L"Transition 1" ) ;
      }
      else 
      {
         state.reset( S_6_FAILED ) ;
         Trace( L"Transition 2" ) ;
      }

      break ;
   case S_5_SUCCEEDED:
      break ;
   case S_6_FAILED:
      break ;
   }
}

void A3_InitAutomaton::OnStateChanged() 
{
   using namespace init_automaton ;
   switch( GetState())
   {
   case S_1_WAIT4USER:

      object_.z21() ;

      break ;
   case S_2_CHECKUSER:

      object_.z22() ;

      break ;
   case S_3_WAIT4PASW:

      object_.z23() ;

      break ;
   case S_4_CHECKPASW:

      object_.z26() ;

      break ;
   case S_5_SUCCEEDED:

      object_.z25() ;

      break ;
   case S_6_FAILED:

      object_.z24() ;

      break ;
   }
}

//////////////////////////////////////////////////////////////////////////
//
// Data Transfer Automaton 
//

A4_DataTransfer::A4_DataTransfer( UserSite& us  )
   : Base      ( datatransf_automaton::S_1_PREPAREDATA ) 
   , object_   ( us )
{
}

BOOL A4_DataTransfer::IsInputEvent( Event event )
{
   return event >= E_30_DC_OPENED && event <= E_34_ALLTRANSFERED || event == E_13_CMD_ABOR ;
}

void A4_DataTransfer::CalcState( OptState& state )
{
   using namespace datatransf_automaton ;

   switch( GetState())
   {
   case S_1_PREPAREDATA:
      
      if( e( 32 ))
      {
         object_.z32() ;
         state.reset( S_2_CREATING_DC ) ;
         Trace( L"Transition 1" ) ;
      }
      else if ( e( 33 ))
      {
         object_.z33() ;
         state.reset( S_4_COMPLETED ) ;
         Trace( L"Transition 2" ) ;
      }
      
      break;
   case S_2_CREATING_DC:

      if( e( 30 ))
      {
         state.reset( S_3_SENDING ) ;
         Trace( L"Transition 1" ) ;
      }
      else if ( e( 31 ))
      {
         object_.z35() ;
         state.reset( S_4_COMPLETED ) ;
         Trace( L"Transition 2" ) ;
      }
      else if ( e( 13 ))
      {
         object_.z38() ;
         object_.z36() ;
         state.reset( S_4_COMPLETED ) ;
         Trace( L"Transition 3" ) ;
      }

      break;
   case S_3_SENDING:
      
      if( e( 34 ))
      {
         object_.z36() ;
         state.reset( S_4_COMPLETED ) ; 
         Trace( L"Transition 1" ) ;
      }
      else if ( e( 13 ))
      {
         object_.z38() ;
         object_.z36() ;
         state.reset( S_4_COMPLETED ) ;
         Trace( L"Transition 2" ) ;
      }
      else if ( e ( 31 ))
      {
         object_.z35() ;
         state.reset( S_4_COMPLETED ) ;
         Trace( L"Transition 3" ) ;
      }

      break;
   case S_4_COMPLETED:

      // nothing 
      
      break;

   }
}

void A4_DataTransfer::OnStateChanged()
{
   using namespace datatransf_automaton ;

   switch( GetState())
   {
   case S_1_PREPAREDATA:

      object_.z30() ;

      break;
   case S_2_CREATING_DC:

      object_.z31() ;

      break;
   case S_3_SENDING:

      object_.z34() ;

      break;
   case S_4_COMPLETED:

      object_.z37() ;

      break;

   }
}

//////////////////////////////////////////////////////////////////////////
//
// Main Automaton 
//

A2_MainAutomaton::A2_MainAutomaton( UserSite& us )
   : Base      ( main_automaton::S_1_INITIALIZATION )
   , object_   ( us )
   , initAuto_ ( us )
   , tranfAuto_( us )
{
}

void A2_MainAutomaton::Init( ITracer * tracer )
{
   Base::Init( tracer ) ;
   
//    initAuto_ .Init( tracer ) ;
//    tranfAuto_.Init( tracer ) ;
}

BOOL A2_MainAutomaton::OnEvent( Event event ) 
{
   BOOL eventProcessed = FALSE ;
   
   if( event >= E_3_TRANSFCOMPLETED && event <= E_19_RETRCOMPLETED )
   {
      eventProcessed = TRUE ;
      event_.reset( event ) ;
      CalcState() ;
   }
   else 
   {
      if ( initAuto_ .ProcessEvent( event ))
         eventProcessed = TRUE ;

      if ( tranfAuto_.ProcessEvent( event ))
         eventProcessed = TRUE ;
   }

   return eventProcessed ;
}

void A2_MainAutomaton::CalcState( OptState& state ) 
{
   using namespace main_automaton ;

   switch( GetState())
   {
   case S_1_INITIALIZATION:

      if( e( 16 ))
      {
         state.reset( S_2_READY ) ;
         Trace( L"Transition 1" ) ;
      }
      else if( e( 17 ))
      {
         state.reset( S_5_CLOSING ) ;
         Trace( L"Transition 2" ) ;
      }

      break ;
   case S_2_READY:
      
      if( e( 12 ))
      {
         object_.z9() ;
         Trace( L"Transition 1" ) ;
      }
      else if( e( 4 ))
      {
         object_.z11() ;
         state.reset( S_5_CLOSING ) ;
         Trace( L"Transition 2" ) ;
      }
      else if( e( 5 ))
      {
         state.reset( S_1_INITIALIZATION ) ;
         object_.z2() ;
         Trace( L"Transition 3" ) ;
      }
      else if( e( 6 ))
      {
         object_.z3() ;
         Trace( L"Transition 4" ) ;
      }
      else if( e( 7 ))
      {
         object_.z4() ;
         Trace( L"Transition 5" ) ;
      }
      else if( e( 8 ))
      {
         object_.z5() ;
         Trace( L"Transition 6" ) ;
      }
      else if( e( 9 ))
      {
         object_.z6() ;
         Trace( L"Transition 7" ) ;
      }
      else if( e( 10 ))
      {
         object_.z7() ;
         Trace( L"Transition 8" ) ;
      }
      else if( e( 11 ))
      {
         object_.z8() ;
         Trace( L"Transition 9" ) ;
      }
      else if( e( 14 ))
      {
         state.reset( S_4_DATATRANSFERING ) ;
         Trace( L"Transition 10" ) ;
      }
      else if( e( 18 ))
      {
         state.reset( S_5_CLOSING ) ;
         Trace( L"Transition 11" ) ;
      }
      else if( e( 15 ))
      {
         object_.z12() ;
         state.reset( S_3_FILETRANSFERING ) ;
         Trace( L"Transition 1" ) ;
      }
      break ;
   case S_3_FILETRANSFERING:
      
      if( e( 13 ))
      {
         object_.z13() ;
         state.reset( S_3_FILETRANSFERING ) ;
         Trace( L"Transition 1" ) ;
      }
      else if( e( 4 ))
      {
         object_.z10() ;
         object_.z11() ;
         state.reset( S_3_FILETRANSFERING ) ;
         Trace( L"Transition 2" ) ;
      }
      else if( e( 18 ))
      {
         object_.z10() ;
         state.reset( S_3_FILETRANSFERING ) ;
         Trace( L"Transition 3" ) ;
      }
      else if ( e( 19 ) && !object_.x1())
      {
         state.reset( S_2_READY ) ;
         Trace( L"Transition 4" ) ;
      }
      else if ( e( 19 ) && object_.x1())
      {
         state.reset( S_5_CLOSING ) ;
         Trace( L"Transition 5" ) ;
      }
      
      break ;
   case S_4_DATATRANSFERING:
      
      if ( e( 3 ) && !object_.x1())
      {
         state.reset( S_2_READY ) ;
         Trace( L"Transition 1" ) ;
      }
      else if( e( 3 ) && object_.x1())
      {
         state.reset( S_5_CLOSING ) ;
         Trace( L"Transition 2" ) ;
      }
      else if( e( 4 ))
      {
         object_.z10() ;
         object_.z11() ;
         state.reset( S_4_DATATRANSFERING ) ;
         Trace( L"Transition 3" ) ;
      }
      else if( e( 18 ))
      {
         object_.z10() ;
         state.reset( S_4_DATATRANSFERING ) ;
         Trace( L"Transition 4" ) ;
      }

      break ;
   case S_5_CLOSING:
      break ;
   }
}

void A2_MainAutomaton::OnStateChanged()
{
   using namespace main_automaton ;

   switch( GetState())
   {
   case S_1_INITIALIZATION :
      
      initAuto_.Init( GetTracer()) ;

      break ;
   case S_2_READY : 
      break ;
   case S_3_FILETRANSFERING :
      break ;
   case S_4_DATATRANSFERING :

      tranfAuto_.Init( GetTracer()) ; 

      break ;
   case S_5_CLOSING :
      
      object_.z1() ;

      break ;
   }
}


} // namespace user_site  