//////////////////////////////////////////////////////////////////////////
//
// UserAutomata.h 
// Provides A2_MainAutomaton, A3_InitAutomaton and A4_DataTransfer declaration, 
// see UserAutomata.cpp for definition 
//
//////////////////////////////////////////////////////////////////////////

#pragma once 
#include <boost/scoped_ptr.hpp>
#include "BaseAutomaton.h"

//////////////////////////////////////////////////////////////////////////
//
// User site automata 
//

class UserSite ;

namespace user_site
{

namespace main_automaton
{
   enum State
   {
      S_1_INITIALIZATION = 1,
      S_2_READY             ,
      S_3_FILETRANSFERING   ,
      S_4_DATATRANSFERING   ,
      S_5_CLOSING             
   } ;
} // main_automaton 

namespace init_automaton
{
   enum State
   {
      S_1_WAIT4USER = 1 ,
      S_2_CHECKUSER     ,
      S_3_WAIT4PASW     ,
      S_4_CHECKPASW     ,
      S_5_SUCCEEDED     ,
      S_6_FAILED        
   };
} // init_automaton 

namespace datatransf_automaton
{
   enum State
   {
      S_1_PREPAREDATA   = 1,
      S_2_CREATING_DC      ,
      S_3_SENDING          ,
      S_4_COMPLETED     
   };
}

   enum Event
   {
      E_1_CMD_USER      = 1,   
      E_2_CMD_PASS         ,
      E_3_TRANSFCOMPLETED  ,
      E_4_CMD_QUIT         ,
      E_5_CMD_REIN         ,
      E_6_CMD_CWD          ,
      E_7_CMD_PWD          ,
      E_8_CMD_PORT         ,
      E_9_CMD_TYPE         ,
      E_10_CMD_STRU        ,
      E_11_CMD_MODE        ,
      E_12_CMD_NOOP        ,
      E_13_CMD_ABOR        ,
      E_14_CMD_LIST        ,
      E_15_CMD_RETR        ,
      
      E_16_INITSUCC        ,
      E_17_INITFAILED      ,

      E_18_QUITREQUEST     ,
      E_19_RETRCOMPLETED   ,

      E_30_DC_OPENED  = 30 ,
      E_31_DC_ERROR        ,
      E_32_DATAREADY       ,
      E_33_DATAERROR       ,
      E_34_ALLTRANSFERED   ,
      E_35_CMD_ABORT
   } ;

struct A2_MainAutomaton ;

struct A3_InitAutomaton
   : public BaseAutomaton< init_automaton::State, Event > 
   {
      typedef BaseAutomaton< init_automaton::State, Event > Base ;

      A3_InitAutomaton( UserSite& us ) ;
      LPCWSTR GetPrefix() const { return L"User Init Automaton: " ; }

   private:
      void CalcState     ( OptState& state ) ;
      void OnStateChanged() ;
      BOOL IsInputEvent  (Event event ) ;

   private:
      UserSite&         object_   ;
   };

struct A4_DataTransfer
   : public BaseAutomaton< datatransf_automaton::State, Event >
{
   typedef BaseAutomaton< datatransf_automaton::State, Event > Base ;

   A4_DataTransfer( UserSite& us ) ;
   LPCWSTR GetPrefix() const { return L"User Data Transfer Automaton: " ; }

private:
   void CalcState     ( OptState& state ) ;
   void OnStateChanged() ;
   BOOL IsInputEvent  (Event event ) ;

private:
   UserSite&         object_   ;
};

struct A2_MainAutomaton
   : public BaseAutomaton< main_automaton::State, Event > 
{
   typedef BaseAutomaton< main_automaton::State, Event > Base ;

   A2_MainAutomaton( UserSite& us ) ;
   LPCWSTR GetPrefix() const { return L"User Main Automaton: " ; }

   void Init      ( ITracer* tracer )  ; // overridden for sub automaton initialization 
   BOOL OnEvent   ( Event event )      ; // overridden for events dispatching 
   
private:
   using Base::CalcState ;

private:
   void CalcState     ( OptState& state ) ;
   void OnStateChanged() ;

private:
   UserSite&        object_   ;

   // inner automata 
   A3_InitAutomaton initAuto_ ;
   A4_DataTransfer  tranfAuto_;
};

} // namespace user_site 

