#include "stdafx.h"

#include "UserSite.h"
#include "CentralPoint.h"
#include "Replies.h"

UserSite :: UserSite( SOCKET sock, CCentralPoint& centralPoint, ILinksProvider* provider, DWORD Id, ITracerManager* mng )
   : TelnetReceiver  ( sock         )
   , cp_             ( centralPoint )
   , links_          ( provider     )
   , authen_         ( provider     )
   , id_             ( Id           )
   , automaton_      ( *this        )

   , needPasw_       ( TRUE  )
   , quiting_        ( FALSE )

   , folder_         ( "/" )
   , tracerMng_      ( mng )
{
   USES_CONVERSION ;

   char buf [0x40] ;
   GetSocketName( &transfParams_.addr ) ;
   transfParams_.addr.GetText( 0x39, buf ) ;

   // Tracer Initialization 
   wchar_t     log [ 0x80 ] ;
   
   string addr = A2W( buf ) ;
   size_t pos = addr.find_first_of( L":" ) ;
   if ( pos != std::string::npos )
      addr[ pos ] = L'-' ;

   swprintf( log, L"session_%d_%s.log", Id, addr.c_str()) ;
   TracerImpl::Init( mng, log ) ;

   Trace( L"Computer connected: %s; connection id: %d", A2W( buf ), id_ ) ;

   CommandParser::Init() ;
   automaton_.Init( GetTracer()) ;
}
   
UserSite ::~UserSite() 
{
}

void UserSite::Stop()
{
   automaton_.ProcessEvent( user_site::E_18_QUITREQUEST ) ;
}

//////////////////////////////////////////////////////////////////////////
//
// automaton support 
//

BOOL UserSite :: x1 () const { return quiting_  ; } // quiting marker  

BOOL UserSite :: x20() const { return corUser_  ; } // user okay  
BOOL UserSite :: x21() const { return needPasw_ ; } // need password 
BOOL UserSite :: x22() const { return corPasw_  ; } // password okay 

void UserSite :: z1() // drop connection 
{
   cp_.UserDropped( id_ ) ;
} 

void UserSite :: z2() // reinit paramters 
{
   transfParams_ = TransferParamters() ;
   SendReply( rep220()) ;
} 
void UserSite :: z3() // proceed CWD 
{
   USES_CONVERSION ;
   std::string folder = lastCmd_.arg ;
   
   if( folder.empty())
   {
      SendReply( rep501()) ;
      return ;
   }

   std::string oldFolder = folder_ ;
   
   if( folder[0] == '/' )    // absolute path 
      folder_ = folder ;
   else if ( folder == ".." ) // the same as CDUP 
   {
      if ( folder_ != "/" )
      {
         size_t pos = folder_.find_last_of( "/" ) ;
         if( pos != std::string::npos )
            folder_.erase( pos + (( pos == 0 ) ? 1 : 0 )) ;
      }
   }
   else // relative path 
      folder_ += (( folder_ != "/" ) ? ( "/" + folder ) : folder ) ;

   if ( folder_ != "/" )
   {
      ILinkPtr link = links_->Search( A2W( folder_.c_str())) ;
      if ( link == NULL || !link->IsFolder())
      {
         folder_ = oldFolder ;
         SendReply( rep501()) ;
         return ;
      }
   }

   SendReply( rep250()) ;
} 

void UserSite :: z4() // proceed PWD 
{ 
   char buf [ 0x400 ] ;
   SendReply( rep257( buf, folder_.c_str())) ; 
}

void UserSite :: z5() // proceed PORT 
{
   USES_CONVERSION ;

   unsigned char addr [ 6 ] ;
   char          line [0x100] ;
   strcpy( line, lastCmd_.arg.c_str()) ;
   
   LPSTR  p = line ;
   size_t count = 0 ;
   while( p != 0 )
   {
      addr[ count++ ] = atoi( p ) ;
      p = strchr( p, ',' );

      if( p != 0 ) 
         ++p ;
   }

   short port = addr[4] * 256 + addr[ 5 ] ;

   transfParams_.addr = TINETAddr( port, addr[0], addr[1], addr[2], addr[3] ) ;
   SendReply(  rep200()) ;
} 

void UserSite :: z6() // proceed TYPE
{
   USES_CONVERSION ;
   transfParams_.type = lastCmd_.arg.c_str()[ 0 ] ;
   SendReply(  rep200()) ;
} 

void UserSite :: z7() // proceed STRU 
{
   USES_CONVERSION ;
   transfParams_.stru = lastCmd_.arg.c_str()[ 0 ] ;
   SendReply(  rep200()) ;
} 
void UserSite :: z8() // proceed MODE
{
   USES_CONVERSION ;
   transfParams_.mode = lastCmd_.arg.c_str()[ 0 ] ;
   SendReply(  rep200()) ;
} 

void UserSite :: z9() // proceed NOOP
{
   SendReply(  rep200()) ;
} 

void UserSite :: z10() // mark quit 
{
   quiting_ = TRUE ;
}
void UserSite :: z11() // reply on QUIT cmd 
{
   SendReply( rep221()) ;
}

void UserSite :: z12() // wake up remote server 
{   
   USES_CONVERSION ;

   std::string file = lastCmd_.arg ;
   if ( file[ 0 ] != '/' )
      file = folder_ + ( folder_ == "/" ? "" : "/" ) + file ;

   ILinkPtr link = links_->Search( A2W( file.c_str())) ;
   if ( link == NULL )
   {
      SendReply( rep501()) ;
      automaton_.ProcessEvent( user_site::E_19_RETRCOMPLETED ) ;
      return ;   
   }

   string ftp_name = link->GetFtpServer() ;
   
   wchar_t user [ 0x100 ] ;
   wchar_t pass [ 0x100 ] ;
   size_t  size = 0xff ; 

   if ( !links_->GetFtpAccount( ftp_name, user, pass, size ))
      return ;

   std::string ftp  = W2A( ftp_name.c_str()) ;
   std::string path = W2A( link->GetFtpLink().c_str()) ;

   size_t pos = ftp.find_first_of( ":" ) ;
   short port = 21 ;
   
   if ( pos != std::string::npos ) 
   {
      std::string port_str ( ftp, pos + 1 ) ;
      port = atoi( port_str.c_str()) ;
   }

   ftp.erase( pos ) ;
   long ip = inet_addr( ftp.c_str()) ;

   TINETAddr addr( port, ip ) ;

   RemoteServers::iterator it = servers_.find( addr ) ;
   if ( it != servers_.end())
      it->second->Download( path.c_str()) ;
   else 
      servers_[ addr ].reset( new RemoteServer( *this, tracerMng_, addr, W2A( user ), W2A( pass ), path.c_str())) ; 
 
   lastUsedServer_ = addr ;
}
void UserSite :: z13() // abort retrieving 
{
   RemoteServers::iterator it = servers_.find( lastUsedServer_ ) ;
   if ( it != servers_.end())
      it->second->Abort() ;      
}

//////////////////////////////////////////////////////////////////////////
//
// initialization 
//

void UserSite :: z21() // send "hello-reply" 
{ 
   SendReply( rep220()) ; 
} 
void UserSite :: z22() // check user 
{
   USES_CONVERSION ;
   std::wstring user = A2W( lastCmd_.arg.c_str()) ;
   IAuthentication::USER_PRIVILEGE up = authen_->CheckUser( user ) ; 

   needPasw_ = up == IAuthentication::UP_NEED_PASW ;
   if( up != IAuthentication::UP_NO_USER )
   {
      user_    = user ;
      corUser_ = TRUE ;
      return ;
   }

   corUser_ = FALSE ;
} 
void UserSite :: z23() { SendReply( rep331()) ; } // send user ok_and_ask_passw reply 
void UserSite :: z24() // send log-in failed  
{ 
   automaton_.ProcessEvent( user_site::E_17_INITFAILED ) ;
   SendReply( rep530()) ; 
} 
void UserSite :: z25() // send user logged-in
{ 
   automaton_.ProcessEvent( user_site::E_16_INITSUCC ) ;
   SendReply( rep230()) ;
} 
void UserSite :: z26() // check password 
{
   USES_CONVERSION ;
   corPasw_ = authen_->CorrectPassw( user_, A2W( lastCmd_.arg.c_str())) ; 
} 


//////////////////////////////////////////////////////////////////////////
//
// data transfering 
//

void UserSite :: z30() 
{
   LPCSTR months[ 13 ] = { "non", "Jan", "Feb", "Mar", "Apr", "May", "Jun",
                                  "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" } ;

   USES_CONVERSION ;
   ILinksIteratorPtr it = links_->CreateItemIterator( A2W( folder_.c_str())) ;
   
   data_.clear() ;

   if( it )
   {
      SYSTEMTIME st ;
      GetSystemTime( &st ) ;

      ILinkPtr link = it->Next() ;
      while ( link != NULL )
      {
         //////////////////////////////////////////////////////////////////////////
         //
         // file list construction 


         // authentication 
         std::string line = ( link->IsFolder()) 
                     ? ( "dr-xr-xr-x 1 owner group " )
                     : ( "-r--r--r-- 1 owner group " ) ;

         // size, right aligned, 13 bytes 
         char size [ 20 ] ; 
         sprintf( size, "%13d ", link->GetSize()) ;
         line += size ;

         // date 
         short year, month, date, hour, minute ; 
         link->GetTime( year, month, date, hour, minute ) ;

         char date_str [ 20 ] ;
         sprintf( date_str, "%s %d ", months[ month ], date ) ;
         line += date_str ;

         if ( year == st.wYear )
            sprintf( date_str, "%d:%d ", hour, minute ) ;
         else 
            sprintf( date_str, "%d ", year ) ;

         line += date_str ;

         // image name 
         line += W2A( link->GetImageName().c_str()) ;
         line += "\r\n" ;
         
         //
         //////////////////////////////////////////////////////////////////////////

         data_ += line ;
         link = it->Next() ;
      }

      automaton_.ProcessEvent( user_site::E_32_DATAREADY ) ;
   }
   else 
      automaton_.ProcessEvent( user_site::E_33_DATAERROR ) ;
}

void UserSite :: z31() 
{
   dc_.reset( new DataConnection( automaton_ )) ;
   dc_->ConnectTo( transfParams_.addr ) ;
}

void UserSite :: z32()
{
   SendReply( rep150()) ;
}
void UserSite :: z33() 
{
   SendReply( rep450()) ;
}
void UserSite :: z34() 
{
   // consider string as continuous buffer 
   dc_->Send( data_.length(), &data_[0] ) ;
}
void UserSite :: z35() 
{
   SendReply( rep425()) ;
   dc_.reset() ;
}
void UserSite :: z36() 
{
   SendReply( rep226()) ;
   dc_.reset() ;
}
void UserSite :: z37() 
{
   automaton_.ProcessEvent( user_site::E_3_TRANSFCOMPLETED ) ;
}

void UserSite :: z38() 
{
   SendReply( rep426()) ;
}

//////////////////////////////////////////////////////////////////////////
void UserSite :: ProcStartRetr( LPCSTR reply )
{
   SendReply( reply ) ;
}
void UserSite :: ProcStopRetr( LPCSTR reply )  
{
   SendReply( reply ) ;
   automaton_.ProcessEvent( user_site::E_19_RETRCOMPLETED ) ;
}
void UserSite :: Retranslate( LPCSTR reply )
{
   SendReply( reply ) ;  
}
void UserSite :: DropRS( TINETAddr const& addr ) 
{
   if ( removedServers_.empty())
      SetTimer( 500 ) ;

   removedServers_.push_back( addr ) ;
}

DWORD UserSite :: ID() const 
{
   return id_ ;
}

TransferParamters const& UserSite :: TransfParams() const 
{
   return transfParams_ ;
}

//////////////////////////////////////////////////////////////////////////

void UserSite::TimerCallback()
{
   RemoveRSRefs() ;
   KillTimer   () ;
}

//////////////////////////////////////////////////////////////////////////
void UserSite :: RemoveRSRefs() 
{
   for ( size_t i = 0, size = removedServers_.size(); i < size; ++i ) 
   {
      RemoteServers::iterator it = servers_.find( removedServers_[ i ] ) ;
      if( it != servers_.end())
         servers_.erase( it ) ;
   }

   removedServers_.clear() ;
}

//////////////////////////////////////////////////////////////////////////

void UserSite :: SendReply( LPCSTR rep ) 
{
   Send ( strlen( rep ), rep ) ;   
   Trace(( string( L"-->" ) + FormatReply( rep )).c_str()) ;
}

void UserSite :: FragmentReceived( int size, const void * data ) 
{
   Trace(( string( L"<-- " ) 
      + CommandParser::Format( ( size_t ) size, ( LPCSTR ) data ).c_str()).c_str()) ;

   if( CommandParser::Parse( ( size_t ) size, ( LPCSTR ) data , lastCmd_ ))
   {
      switch( lastCmd_.cmd )
      {
      case CommandParser::USER :
         automaton_.ProcessEvent( user_site::E_1_CMD_USER ) ;
         break ;  
      case CommandParser::PASS :
         automaton_.ProcessEvent( user_site::E_2_CMD_PASS ) ;
         break ;
      case CommandParser::QUIT :
         automaton_.ProcessEvent( user_site::E_4_CMD_QUIT ) ;
         break ;
      case CommandParser::REIN :
         automaton_.ProcessEvent( user_site::E_5_CMD_REIN ) ;
         break ;
      case CommandParser::CWD  :
         automaton_.ProcessEvent( user_site::E_6_CMD_CWD ) ;
         break ;
      case CommandParser::PWD :
         automaton_.ProcessEvent( user_site::E_7_CMD_PWD ) ;
         break ;
      case CommandParser::PORT :
         automaton_.ProcessEvent( user_site::E_8_CMD_PORT ) ;
         break ;
      case CommandParser::TYPE :
         automaton_.ProcessEvent( user_site::E_9_CMD_TYPE ) ;
         break ;
      case CommandParser::STRU :
         automaton_.ProcessEvent( user_site::E_10_CMD_STRU ) ;
         break ;
      case CommandParser::MODE :
         automaton_.ProcessEvent( user_site::E_11_CMD_MODE ) ;
         break ;
      case CommandParser::ABOR :
         automaton_.ProcessEvent( user_site::E_13_CMD_ABOR ) ;
         break ;  
      case CommandParser::NOOP :
         automaton_.ProcessEvent( user_site::E_12_CMD_NOOP ) ;
         break ;
      case CommandParser::LIST :
         automaton_.ProcessEvent( user_site::E_14_CMD_LIST ) ;
         break ;
      case CommandParser::RETR :
         automaton_.ProcessEvent( user_site::E_15_CMD_RETR ) ;
         break ;
      }
   }
   else 
      SendReply( rep202()) ; // not implemented
}

void UserSite :: ConnectChanged  ()
{
   if ( !connected )
      cp_.UserDropped( id_ ) ;
}
