#pragma once 
#include "UserAutomata.h"
#include "Command.h"
#include "Transfer.h"
#include "DataConnection.h"
#include "Telnet.h"
#include "RemoteServer.h"

#include <boost\shared_ptr.hpp>

class CCentralPoint ;

class UserSite 
   : public    TracerImpl
   , protected TelnetReceiver
{
public:
   typedef boost::shared_ptr< UserSite >  PUserSite ;

public:
   UserSite( SOCKET sock, CCentralPoint& centralPoint, ILinksProvider* provider, DWORD Id, ITracerManager* mng ) ;
  ~UserSite() ;

public:
   void Stop() ;


// automaton support 
public:

   BOOL x1()  const ; // quiting

   BOOL x20() const ; // user okay 
   BOOL x21() const ; // need password 
   BOOL x22() const ; // password okay 

   // for main user site automaton 
   void z1() ; // drop connection 
   void z2() ; // reinit paramters 
   void z3() ; // proceed CWD 
   void z4() ; // proceed PWD
   void z5() ; // proceed PORT
   void z6() ; // proceed TYPE
   void z7() ; // proceed STRU
   void z8() ; // proceed MODE
   void z9() ; // proceed NOOP
   void z10() ; // mark quit 
   void z11() ; // reply on QUIT command 
   void z12() ; // wake up remote server 
   void z13() ; // abort retrieving 

   // for initialization automaton 
   void z21() ; // send "hello-reply"
   void z22() ; // check user
   void z23() ; // send user ok_and_ask_passw reply 
   void z24() ; // send log-in failed 
   void z25() ; // send user loged-in
   void z26() ; // check user

   // for data transfering ( mostly LIST data )
   void z30() ;
   void z31() ;
   void z32() ;
   void z33() ;
   void z34() ;
   void z35() ;
   void z36() ;
   void z37() ;   
   void z38() ;

// from remote server 
public:
   void  ProcStartRetr( LPCSTR reply ) ; 
   void  ProcStopRetr ( LPCSTR reply ) ; 
   void  Retranslate  ( LPCSTR reply ) ;
   void  DropRS       ( TINETAddr const& addr ) ;
   DWORD ID           () const ;

   TransferParamters const& TransfParams() const ;

// for tracer 
public:
   LPCWSTR GetPrefix() const { return L"UserSite: " ; }

private:
   typedef 
      std::map< TINETAddr, boost::shared_ptr< RemoteServer > > 
      RemoteServers ;

private:
   void SendReply       ( LPCSTR rep ) ;
   void FragmentReceived( int size, const void * data ) ;
   void ConnectChanged  () ;

private:
   void TimerCallback() ;

private:
   void RemoveRSRefs() ;

private:
   CCentralPoint&       cp_      ; 
   ILinksProviderPtr    links_   ;
   IAuthenticationPtr   authen_  ;
   DWORD                id_      ;

   CommandParser::Command  lastCmd_   ;

// tracer
private:
   ITracerManagerPtr tracerMng_ ;

// automaton 
private:
   user_site::A2_MainAutomaton   automaton_ ;

// remote servers 
   RemoteServers  servers_        ;
   TINETAddr      lastUsedServer_ ;
   
   std::vector< TINETAddr > removedServers_ ;

// initialization 
   std::wstring user_     ;
   BOOL         needPasw_ ;
   BOOL         corUser_  ;
   BOOL         corPasw_  ;

// transfer 
   boost::scoped_ptr< DataConnection > dc_   ;
   std::string                         data_ ;

   TransferParamters transfParams_ ;

// working folder 
   std::string       folder_ ;

// quiting 
   BOOL quiting_ ;
};