#include "stdafx.h"
#include "transport.h"

#define WM_TSOCKET_EVENT (WM_USER + 1000)
#define WM_SEARCHHOST    (WM_USER + 1001) 

inline void * new_mem ( int size, const void * data )
{
   void * t = 0 ;
   if ( size )
   {
        t = new char [ size ] ;
        memcpy ( t, data, size ) ;
   }
   return t ;
}
// -------------------------------------------------------------------

TPointName :: TPointName () : size ( 0 ), data ( 0 ) {}
TPointName :: TPointName ( int size, const void * data ) :
    size ( 0 ), data ( 0 )
{
   SetData ( size, data ) ;
}
TPointName :: TPointName ( const TPointName& name ) :
    size ( 0 ), data ( 0 )
{
   SetData ( name.size, name.data ) ;
}
TPointName ::~TPointName ()
{
   delete data ;
}
void TPointName :: SetData ( int _size, const void * _data )
{
   if ( size == _size )
        memcpy ( data, _data, size ) ;     // more efficient
   else
   {
        size = _size ;
        void * t = data ;                  // to avoid self copy
        data = new_mem ( size, _data ) ;
        delete t ;
   }
}

/*
char * TPointName :: GetText ( int bufSize, char * buf )
{
   DWORD sz = bufSize ;
   *buf = 0 ;
   if ( data ) WSAAddressToString ( (PSOCKADDR)data, size, NULL, buf, & sz ) ;
   return buf ;
}
*/

// -------------------------------------------------------------------------
#define mask (*(sockaddr_in *)data)
#define un   (mask.sin_addr.S_un)
short&  TINETAddr :: family () { return mask.sin_family ; }
int&    TINETAddr :: ipaddr () { return (int&) (un.S_addr) ; }
int     TINETAddr :: getip  ()  const { return (un.S_addr) ; }
ushort  TINETAddr :: port   ()  const { return ntohs(mask.sin_port) ; }
void    TINETAddr :: setport( ushort port ) { mask.sin_port = htons ( port ) ; }

TINETAddr :: TINETAddr () : TPointName ()
{
   size = sizeof ( sockaddr_in ) ;
   data = new char [ size ] ;
   memset ( data, 0, size ) ;
   family() = AF_INET ;
}
TINETAddr :: TINETAddr ( ushort port, int addr ): TPointName ()
{
   size = sizeof ( sockaddr_in ) ;
   data = new char [ size ] ;
   memset ( data, 0, size ) ;
   family() = AF_INET ;
   ipaddr() = addr ;
   setport( port ) ; 
}
TINETAddr :: TINETAddr ( ushort port, char b1, char b2, char b3, char b4 ) : TPointName ()
{
   size = sizeof ( sockaddr_in ) ;
   data = new char [ size ] ;
   memset ( data, 0, size ) ;
   family() = AF_INET ;
   setport( port ) ;
   un.S_un_b.s_b1 = b1 ; un.S_un_b.s_b2 = b2 ; un.S_un_b.s_b3 = b3 ; un.S_un_b.s_b4 = b4 ;
}

char * TINETAddr :: GetText ( int sz, char * buf ) const
{
   char b [ 50 ] ;
   sprintf ( b, "%d.%d.%d.%d:%d", 
                un.S_un_b.s_b1, un.S_un_b.s_b2,
                un.S_un_b.s_b3, un.S_un_b.s_b4, port()) ;
   strncpy ( buf, b, sz ) ;
   buf [ sz - 1 ] = 0 ;
   return buf ;
}

// -------------------------------------------------------------------
TNetMessage * TNetMessage :: Create  ( int size, const void * data, const TPointName * dest)
{
   TNetMessage * t = ( TNetMessage * ) new char [ size ] ;
   t -> dest = dest ? new TPointName ( *dest ) : NULL ;
   t -> size = size ;
   memcpy ( t -> data, data, size ) ;
   return t ;
}

void destroy ( TNetMessage * msg )
{
   if ( msg )
   {
        if ( msg -> dest ) delete (msg -> dest) ;
        delete (char*)msg ;
   }
}

// -------------------------------------------------------------------
void TNetPoint :: ReadySend ()
{
   while ( ! outlist.empty() )
   {
        TNetMessage * msg = outlist.front () ;
        int size = msg -> size - offset ;
        if ( DoSend ( & size, msg -> data + offset, msg -> dest ) )
        {
             offset += size ;
             if ( offset != msg -> size ) return ; // have some more to send, but send was blocked
        }
        offset = 0 ;
        outlist.pop_front () ;
   }

   if ( outlist.empty())
      AllDataSended() ;
}

void TNetPoint :: Clear ()
{
   outlist.erase ( outlist.begin(), outlist.end() ) ;
   offset = 0 ;
}

void TNetPoint :: Send ( int size, const void * data, const TPointName * name )
{
   sends ++ ;
   if ( outlist.empty () )
   {
        int sz = size ;
        if ( !DoSend ( & sz, data, name )) 
           return ;

        if ( sz == size )
        {
           AllDataSended() ;
           return ;
        }

        data = (char*)data + sz ;
        size -= sz ;
   }
   outlist.push_back ( TNetMessage :: Create ( size, data, name ) ) ;
   faults ++ ;
}

TNetPoint :: ~TNetPoint ()
{
//   DebugMessageEx ( "socket", "sends: %d, queue faults: %d", sends, faults ) ;
}

// -----------------------------------------------------------------  class TSocket
int TSocket :: wsaCounter = 0 ;

void TSocket :: InitWSA ()
{
   if ( ! wsaCounter )
   {
        WORD wVersionRequested;
        WSADATA wsaData;
        wVersionRequested = MAKEWORD( 2, 0 );
        WSAStartup( wVersionRequested, &wsaData ) ;
   }
   wsaCounter ++ ;
}

TCHAR TSocket :: className [] = _T("TSocketWindow") ;

void TSocket :: CreateSockWindow ()
{
   WNDCLASS wclass;
   HINSTANCE hInstance = GetModuleHandle ( 0 ) ; // _Module.GetModuleInstance () ;

   if ( ! ::GetClassInfoA( hInstance, className, &wclass ) )
   {
        wclass.style         = 0;
        wclass.lpfnWndProc   = WndProc;
        wclass.cbClsExtra    = NULL;
        wclass.cbWndExtra    = NULL;
        wclass.hInstance     = hInstance;
        wclass.hIcon         = NULL;
        wclass.hCursor       = NULL;
        wclass.hbrBackground = 0;
        wclass.lpszMenuName  = (LPSTR) NULL;
        wclass.lpszClassName = (LPSTR) className;
        RegisterClass ( & wclass );
   }
   hwnd = CreateWindowEx ( 0, className, className,
                           0, 0, 0, 0, 0, NULL, NULL, hInstance, NULL ) ;
   SetWindowLong ( hwnd, GWL_USERDATA, (LONG) this ) ;
}

long FAR PASCAL TSocket :: WndProc ( HWND hWnd, unsigned msg, WPARAM wParam, LPARAM lParam )
{
   TSocket * s = (TSocket *)GetWindowLong ( hWnd, GWL_USERDATA ) ;
   if ( s )
   {
        if ( msg == WM_TSOCKET_EVENT )
        {
             int err = WSAGETSELECTERROR ( lParam ) ;
             if ( err ) s -> SocketError ( err ) ;
             // else ???
             switch ( WSAGETSELECTEVENT ( lParam ) )
             {
                case FD_READ   : s -> Process_FD_READ   ( err ) ; break ;
                case FD_WRITE  : s -> Process_FD_WRITE  ( err ) ; break ;
                case FD_ACCEPT : s -> Process_FD_ACCEPT ( err ) ; break ;
                case FD_CONNECT: s -> Process_FD_CONNECT( err ) ; break ;
                case FD_CLOSE  : s -> Process_FD_CLOSE  ( err ) ; break ;
             }
             return 1 ;
        }
        if ( msg == WM_SEARCHHOST )
        {
             s -> Process_SearchHost ( WSAGETASYNCERROR(lParam) ) ; 
             return 1 ;
        }

        if ( msg == WM_TIMER && wParam == 1 )
        {
             s -> TimerCallback () ;
             return 1 ;
        }

   }
   return DefWindowProc( hWnd, msg, wParam, lParam ) ;
}

void TSocket :: SetTimer  ( int delay )
{
   KillTimer () ;
   :: SetTimer ( hwnd, 1, delay, NULL ) ;
   timerOn = 1 ;
}

void TSocket :: KillTimer ()
{
   if ( timerOn ) ::KillTimer ( hwnd, 1 ) ;
   timerOn = 0 ;
}

TSocket :: TSocket () : timerOn ( 0 )
{
   InitWSA () ;
   CreateSockWindow () ;
   sock = INVALID_SOCKET ;
}

TSocket :: TSocket ( int af, int type, int protocol ) : timerOn ( 0 )
{
   InitWSA () ;
   CreateSockWindow () ;
   sock = socket ( af, type, protocol ) ;
}

TSocket :: TSocket ( const SOCKET& _sock ) : timerOn ( 0 )
{
   InitWSA () ;
   CreateSockWindow () ;
   sock = _sock ;
}

TSocket :: ~TSocket ()
{
   KillTimer () ;
   if ( sock != INVALID_SOCKET ) closesocket ( sock ) ;
   DestroyWindow ( hwnd ) ;
   if ( -- wsaCounter == 0 ) 
        WSACleanup () ;
}

void TSocket :: SetSocket ( const SOCKET& _sock ) 
{
   if ( sock != INVALID_SOCKET ) closesocket ( sock ) ;
   sock = _sock ;
}

void TSocket :: AsyncSelect ( long event )
{
    WSAAsyncSelect ( sock, hwnd, WM_TSOCKET_EVENT, event ) ;
}

int TSocket :: Bind    ( const TPointName * name )
{
    if ( bind ( sock, (PSOCKADDR)(name -> data), name -> size ) != SOCKET_ERROR ) return 1 ;
    SocketError ( WSAGetLastError () ) ;
    return 0 ;
}

int TSocket :: Connect ( const TPointName * name )
{
    if ( connect ( sock, (PSOCKADDR)(name -> data), name -> size ) != SOCKET_ERROR ) return 1 ;
    int ret = WSAGetLastError () ;
    if ( ret == WSAEWOULDBLOCK ) return 1 ;
    SocketError ( ret ) ;
    return 0 ;
}

int TSocket :: Reset ()
{
    WSAPROTOCOL_INFO info ;
    int sz = sizeof ( info ) ;
    if ( getsockopt ( sock, SOL_SOCKET, SO_PROTOCOL_INFO, (char*)&info, & sz ) != SOCKET_ERROR )
    {
         closesocket ( sock ) ;
         sock = socket ( info.iAddressFamily, info.iSocketType, info.iProtocol ) ;
         if ( sock != INVALID_SOCKET ) return 1 ;
    }
    SocketError ( WSAGetLastError () ) ;
    return 0 ;
}

int TSocket :: Listen  ()
{
    if ( listen ( sock, 50 ) != SOCKET_ERROR ) return 1 ;
    SocketError ( WSAGetLastError () ) ;
    return 0 ;
}

int TSocket :: GetSocketName ( TPointName * name ) const
{
    char buf [ MAX_PATH ] ;
    int bufLen = sizeof ( buf ) ;
    if ( getsockname ( sock, (PSOCKADDR)buf, &bufLen ) != SOCKET_ERROR )
    {
         name -> SetData ( bufLen, buf ) ;
         return 1 ;
    }
    SocketError ( WSAGetLastError () ) ;
    return 0 ;
}
int TSocket :: GetPeerName   ( TPointName * name ) const
{
    char buf [ MAX_PATH ] ;
    int bufLen = sizeof ( buf ) ;
    if ( getpeername ( sock, (PSOCKADDR)buf, &bufLen ) != SOCKET_ERROR )
    {
         name -> SetData ( bufLen, buf ) ;
         return 1 ;
    }
    SocketError ( WSAGetLastError () ) ;
    return 0 ;
}

int TSocket :: RecvFrom ( int * size, void * buf, TPointName * from )
{
    int ret ;
    if ( from )
    {
         char addr [ 100 ] ;
         int  addrLen = sizeof ( addr ) ;
         ret = recvfrom ( sock, (char *)buf, *size, 0, (PSOCKADDR)addr, &addrLen ) ;
         from -> SetData ( addrLen, addr ) ;
    }
    else ret = recv (  sock, (char *)buf, *size, 0 ) ;
    if ( ret == SOCKET_ERROR )
    {
         SocketError ( WSAGetLastError () ) ;
         *size = 0 ;
         return 0 ;
    }
    else
    {
         *size = ret ;
         return 1 ;
    }
}

int TSocket :: SendTo   ( int * size, const void * buf, const TPointName * dest )
{
    int ret = dest ?
               sendto ( sock, (char *)buf, *size, 0, (PSOCKADDR)(dest -> data), dest -> size ) :
               send   ( sock, (char *)buf, *size, 0 ) ;
    if ( ret != SOCKET_ERROR )
    {    *size = ret ; return 1 ; }
    *size = 0 ;
    int err = WSAGetLastError () ;
    if ( err == WSAEWOULDBLOCK ) return 1 ;
    SocketError ( err ) ;
    return 0 ;
}

int TSocket :: SetOption   ( int level, int optname, const void * optval, int optlen ) 
{
    return setsockopt ( sock, level, optname, (const char *)optval, optlen ) ;
}

// -------------------------------------------------------------  TPinger
#pragma pack(push, 4)

struct IpHeader 
{
	unsigned int h_len:4;          // length of the header
	unsigned int version:4;        // Version of IP
	unsigned char tos;             // Type of service
	unsigned short total_len;      // total length of the packet
	unsigned short ident;          // unique identifier
	unsigned short frag_and_flags; // flags
	unsigned char  ttl; 
	unsigned char proto;           // protocol (TCP, UDP etc)
	unsigned short checksum;       // IP checksum

	unsigned int sourceIP;
	unsigned int destIP;

};

struct IcmpHeader {
  BYTE i_type;
  BYTE i_code; /* type sub code */
  USHORT i_cksum;
  USHORT i_id;
  USHORT i_seq;
  /* This is not the std header, but we reserve space for time */
  ULONG timestamp;
};

#pragma pack(pop)

#define ICMP_ECHO      8
#define ICMP_ECHOREPLY 0
#define ICMP_MIN       8 

static ushort checksum ( ushort *buffer, int size ) 
{
  unsigned long cksum=0;

  while ( size > 1 ) 
  {
     cksum += *buffer++;
     size  -= sizeof(ushort);
  }
  
  if( size )
     cksum += *(UCHAR*)buffer;

  cksum = (cksum >> 16) + (cksum & 0xffff);
  cksum += (cksum >>16);

  return (ushort)(~cksum);
}

#define ICMPSIZE  (sizeof(IcmpHeader) + 32)

TPinger :: TPinger () : 
    TSocket    ( AF_INET, SOCK_RAW, IPPROTO_ICMP ), 
    m_bpinging ( FALSE ), 
    m_bNoReply ( FALSE ) 
{
    m_echoid = (ushort)GetCurrentProcessId() ; 

    int timeout = 1000 ; 
    setsockopt ( sock, SOL_SOCKET, SO_RCVTIMEO, (char*)&timeout, sizeof(timeout) );

    timeout = 1000 ;
    setsockopt ( sock, SOL_SOCKET, SO_SNDTIMEO, (char*)&timeout, sizeof(timeout) );

    int b = 1 ;
    setsockopt ( sock, SOL_SOCKET, SO_BROADCAST, (char *) &b, sizeof ( b ) ) ;

    AsyncSelect ( FD_WRITE | FD_READ ) ;

    m_pdata = (IcmpHeader *) new char [ICMPSIZE];

    memset ( m_pdata, 'E', ICMPSIZE ) ; 

    m_pdata -> i_type  = ICMP_ECHO;
    m_pdata -> i_code  = 0;
    m_pdata -> i_id    = m_echoid ;

    memset(&m_dest, 0, sizeof(m_dest) );
    m_dest.sin_family  = AF_INET;
}

TPinger :: ~TPinger () 
{
    KillTimer () ; 
    delete m_pdata ; 
}  

void TPinger :: Process_FD_READ   ( int error ) 
{
    if ( m_bpinging && ! error )
    {
         struct sockaddr_in from;
         int fromlen = sizeof ( from ) ; 

         char buf [256] ; 
         int bread = recvfrom ( sock, buf, sizeof(buf), 0, (struct sockaddr*)&from, &fromlen ) ; 
         if ( bread != 0 ) 
         {
              IpHeader   * iphdr    = (IpHeader *) buf ;
              ushort       iphdrlen = iphdr -> h_len * 4 ; 

              IcmpHeader * icmphdr  = (IcmpHeader*)(buf + iphdrlen) ;

              if ( bread >= iphdrlen + ICMP_MIN        && 
                   icmphdr -> i_type == ICMP_ECHOREPLY && 
                   icmphdr -> i_id == m_echoid         &&
                   m_dest.sin_addr.S_un.S_addr == from.sin_addr.S_un.S_addr )
              {
                   m_bNoReply = FALSE ; 
                   BOOL cancel = TRUE ; 
                   EchoReceived ( &cancel ) ; 
                   if ( cancel ) 
                   {
                        KillTimer () ; 
                        m_bpinging = FALSE ; 
                   }
              }
         }
    }
}

void TPinger :: TimerCallback () 
{
    if ( m_bNoReply ) 
         TimeOut () ; 
    m_pdata -> i_cksum = 0;
    m_pdata -> timestamp = GetTickCount();
    m_pdata -> i_seq = m_seqno++;

    m_pdata -> i_cksum = checksum ( (ushort*)m_pdata, ICMPSIZE ) ;

    sendto ( sock, (char *)m_pdata, ICMPSIZE, 0, (struct sockaddr*)&m_dest, sizeof(m_dest));
    m_bNoReply = TRUE ; 
}

void TPinger :: Process_SearchHost ( int err ) 
{
    if ( m_bpinging ) 
    {
         if ( err )
         {
              if ( ! WSAAsyncGetHostByName ( hwnd, WM_SEARCHHOST, m_host, m_hostent, sizeof ( m_hostent) ) )
                   m_bpinging = FALSE ; 
         }
         else
         {
              m_bNoReply = FALSE ; 
              m_seqno    = 0 ; 
              SetTimer ( 1000 ) ; 
              HOSTENT * hp = (HOSTENT *) m_hostent ; 
              memcpy( &(m_dest.sin_addr), hp -> h_addr, hp -> h_length );
              TimerCallback () ; 
         }
    }
}

void TPinger :: Ping ( const char * hostname ) 
{
    if ( ! m_bpinging ) 
    {
         m_bpinging = TRUE ; 
         strcpy ( m_host, hostname ) ; 

         if ( ! WSAAsyncGetHostByName ( hwnd, WM_SEARCHHOST, m_host, m_hostent, sizeof ( m_hostent ) ) )
              m_bpinging = FALSE ; 
    }
}

// -------------------------------------------------------------  TSockPoint
void TSockPoint :: Process_FD_READ   ( int error )
{
    if ( ! error )
    {
         char buf  [ 1500 ] ;  // !!!!!!!!!!!!!!! check max packet length
         int  size = sizeof ( buf ) ;
         TPointName from ;
         if ( RecvFrom ( &size, buf, sourceEnabled ? &from : NULL ) )
              Receive ( size, buf, sourceEnabled ? &from : NULL  ) ;
    }
}

void TSockPoint :: Process_FD_WRITE  ( int error )
{   if ( ! error ) ReadySend () ; }

int  TSockPoint :: DoSend ( int * size, const void * data, const TPointName * dest )
{   return SendTo ( size, data, dest ) ;  }

void TSockPoint :: Error  ( int code ) const
{ 
    if ( code != 10061 )                
       __TRACE ( L"Socket_Error: Error %d on sock %d", code, sock ) ; 
}

int  TSockPoint :: SetLocalName ( const TPointName& name )
{   return Bind ( &name ) ;  }

int  TSockPoint :: ConnectTo    ( const TPointName& name )
{   return Connect ( &name ) ; }

void TSockPoint :: GetLocalName   ( TPointName& name ) const
{  GetSocketName ( &name ) ; }

void TSockPoint :: GetConnectName ( TPointName& name ) const
{  GetPeerName ( &name ) ; }

// -------------------------------------------------------- TSockDgram

TSockDgram :: TSockDgram ( int af, int protocol ) :
    TSockPoint ( af, SOCK_DGRAM, protocol )
{
    int b = 1 ;
    SetOption   ( SOL_SOCKET, SO_BROADCAST, &b, sizeof ( b ) ) ;
    AsyncSelect ( FD_WRITE | FD_READ ) ;
}

int TSockDgram :: DoSend ( int * size, const void * data, const TPointName * dest ) 
{
    return TSockPoint :: DoSend ( size, data, dest ? dest : (defDest ? &defDest : NULL ) ) ;
}



// ---------------------------------------------------------- TSockStream

void TSockStream :: Process_FD_CONNECT ( int error )
{
    if ( ! error )
    {
         connected = 1 ;
         ConnectChanged () ;
    }
    else SetTimer ( 1000 ) ;
}

void TSockStream :: Process_FD_CLOSE   ( int )
{
    connected = 0 ;
    ConnectChanged () ;
}

void TSockStream :: TimerCallback ()
{
    KillTimer () ;
    TSockPoint :: ConnectTo ( lastConnect ) ;
}

TSockStream :: TSockStream ( int af, int protocol ) :
    TSockPoint ( af, SOCK_STREAM, protocol ), connected ( 0 )
{
    AsyncSelect ( FD_WRITE | FD_READ | FD_CONNECT | FD_CLOSE ) ;
}

TSockStream :: TSockStream ( const SOCKET& sock ) :
    TSockPoint ( sock ), connected ( 1 )
{
    AsyncSelect ( FD_WRITE | FD_READ | FD_CONNECT | FD_CLOSE ) ;
}

void TSockStream :: SetSocket ( const SOCKET& sock ) 
{
    TSocket :: SetSocket ( sock ) ;
    connected = 1 ;
    AsyncSelect ( FD_WRITE | FD_READ | FD_CONNECT | FD_CLOSE ) ;
}

int TSockStream :: ConnectTo    ( const TPointName& name )
{
    lastConnect = name ;
    return TSockPoint :: ConnectTo ( name ) ;
}

int TSockStream :: Reconnect  ()
{
    TPointName local ;
    GetLocalName ( local ) ;
    if ( ! Reset () ) return 0 ;
    Clear () ;
    AsyncSelect ( FD_WRITE | FD_READ | FD_CONNECT | FD_CLOSE ) ;
    SetLocalName ( local ) ;
    return ConnectTo ( lastConnect ) ;
}

int TSockStream :: Shutdown ()
{
    return shutdown ( sock, 2 ) != SOCKET_ERROR;
}

// ------------------------------------------------------- class TFragmentStream
void TFragmentStream :: Receive ( int size, const void * data, const TPointName * )
{
   const short clrf = 0x0a0d ;

   char * p = (char*)data ;
   char * e = p + size ;
   while ( p < e )
     if ( len < 4 ) buf [len++] = *p++;     // get message size
     else
     {
          int msgSize = *(int*)buf ;
          if ( sz < msgSize)
               buf = (char*)realloc ( buf, sz = (msgSize & 0xffffff00) + 256 ) ;
          int cnt = min ( e - p, msgSize - len ) ;
          memcpy ( buf + len, p, cnt ) ;
          p  += cnt ;
          len += cnt ;
          if ( len == msgSize )
          {
               // message ready
               FragmentReceived ( msgSize, buf ) ;
               len = 0 ;
          }
     }
}

int TFragmentStream :: Reconnect  ()
{
    len = 0 ;
    return TSockStream :: Reconnect () ;
}

TFragmentStream :: TFragmentStream (  int af, int protocol ) :
    TSockStream ( af, protocol ), sz ( 500 ), len ( 0 )
{
    buf = new char [ sz ] ;
}
TFragmentStream :: TFragmentStream ( const SOCKET& sock ) :
    TSockStream ( sock ), sz ( 500 ), len ( 0 )
{
    buf = new char [ sz ] ;
}

TFragmentStream :: TFragmentStream () : sz ( 500 ), len ( 0 ) 
{
    buf = new char [ sz ] ;
}

TFragmentStream :: ~TFragmentStream ()
{
    delete buf ;
}

void TFragmentStream :: SetSocket ( const SOCKET& sock ) 
{
    len = 0 ;
    TSockStream :: SetSocket ( sock ) ;
}


// ------------------------------------------------------ class TListener
TListener :: TListener ( int af, int type, int protocol ) :
    TSocket ( af, type, protocol )
{
    AsyncSelect ( FD_ACCEPT ) ;
}

void TListener :: Process_FD_ACCEPT ( int error )
{
    if ( ! error )
    {
         SOCKET s = accept ( sock, NULL, NULL ) ;
         if ( s != INVALID_SOCKET ) Accepted ( s ) ;
    }
}

int TListener :: ListenAs ( const TPointName& name )
{
    return Bind ( &name ) && Listen () ;
}

void TListener :: SocketError ( int code ) const
{
   __TRACE ( L"Socket_Error: Error %d on socket %d", code, sock ) ;
}

