#pragma once 


#ifdef _MSC_VER 
  #pragma warning ( disable : 4200)
  #pragma pack ( push, 1 )
#endif

#include <list>


typedef unsigned short ushort ;


class TPointName
{
  public :
   int    size ;
   void * data ;
   TPointName () ;
   TPointName ( int size, const void * data ) ;
   TPointName ( const TPointName& name ) ;
  ~TPointName () ;

   operator int () const { return data != 0 ; }
   char * GetText ( int size, char * buf ) { *buf = 0; return buf ; }
   void   SetData ( int size, const void * data ) ;
   TPointName& operator = ( const TPointName& name )
   {
      SetData ( name.size, name.data ) ;
      return *this ;
   }
} ;

class TINETAddr : public TPointName
{
  public :
   short&  family () ;
   int&    ipaddr () ;
   int     getip  () const ;
   void    setport( ushort port ) ;
   ushort  port   () const ;
   TINETAddr () ;
   TINETAddr ( ushort port, int addr ) ;
   TINETAddr ( ushort port, char b1, char b2, char b3, char b4 ) ;
   char * GetText ( int size, char * buf ) const ;
   
   TINETAddr& operator = ( const TINETAddr& addr )
   {
      SetData ( addr.size, addr.data ) ;
      return *this ;
   }
} ;

// name == NULL, in case of default destination
struct TNetMessage
{
   TPointName * dest ;
   int          size ;
   char         data [];
   static TNetMessage * Create  ( int size, const void * data, const TPointName * dest = NULL ) ;
} ;

void destroy ( TNetMessage * msg ) ;

typedef std::list<TNetMessage*> TNetMessageList ;

class TNetPoint
{
  private :
   int faults, sends ;
  protected :
   TNetMessageList outlist ;
   int offset ;

   void ReadySend () ;
   virtual int  DoSend  ( int * size, const void * data, const TPointName * dest ) = 0 ;

   virtual void Error ( int code ) const = 0 ;
   virtual void Receive ( int   size, const void * data, const TPointName * from ) = 0 ;
  public:
   TNetPoint () : offset ( 0 ), sends ( 0 ), faults ( 0 ) {}
   virtual ~TNetPoint () ;

   virtual int  SetLocalName ( const TPointName& name ) = 0 ;
   virtual int  ConnectTo    ( const TPointName& name ) = 0 ;
   virtual void GetLocalName   ( TPointName& name ) const = 0 ;
   virtual void GetConnectName ( TPointName& name ) const = 0 ;

   void Clear () ;
   virtual void Send ( int size, const void * data, const TPointName * name = NULL ) ;

   virtual void AllDataSended() {} 
} ;

class TSocket
{
  private :
   static int   wsaCounter ;     
   static TCHAR className [] ;
   static long FAR PASCAL WndProc ( HWND hWnd, unsigned msg, WPARAM wParam, LPARAM lParam ) ;
   static void InitWSA () ;
  protected :
   virtual void TimerCallback   () {}
   virtual void SocketError ( int code ) const = 0 ;
   virtual void Process_FD_READ   ( int error ) {}
   virtual void Process_FD_WRITE  ( int error ) {}
   virtual void Process_FD_ACCEPT ( int error ) {}
   virtual void Process_FD_CONNECT( int error ) {}
   virtual void Process_FD_CLOSE  ( int error ) {}
   virtual void Process_SearchHost( int error ) {}  

  protected :
   HWND   hwnd ;
   SOCKET sock ;
   int    timerOn ;

   void   CreateSockWindow () ;
   void   SetTimer  ( int delay ) ;
   void   KillTimer () ;
  public :
   TSocket () ;
   TSocket ( int af, int type, int protocol ) ;
   TSocket ( const SOCKET& sock ) ;
   virtual ~TSocket () ;

   void SetSocket   ( const SOCKET& sock ) ; 
   void AsyncSelect ( long event ) ;
   int  Reset       () ;
   int  Bind        ( const TPointName * name ) ;
   int  Connect     ( const TPointName * name ) ;
   int  Listen      () ;
   int  SetOption   ( int level, int optname, const void * optval, int optlen ) ; 

   // size on input = sizeof ( buf ), on output = number of bytes ( read or written )
   int  RecvFrom ( int * size,       void * buf,       TPointName * from = 0 ) ;
   int  SendTo   ( int * size, const void * buf, const TPointName * dest = 0 ) ;

   int  GetSocketName ( TPointName * name ) const ;
   int  GetPeerName   ( TPointName * name ) const ;

   SOCKET GetSocket () const { return sock ; }   
} ;

class TSockPoint : public TNetPoint, public TSocket
{
  protected :
   int  sourceEnabled ;
   void Process_FD_READ   ( int error ) ;
   void Process_FD_WRITE  ( int error ) ;
   void SocketError ( int code ) const { Error ( code ) ; }

   void Error  ( int code ) const ;
   int  DoSend ( int * size, const void * data, const TPointName * dest ) ;
  public:
   TSockPoint () : sourceEnabled ( 0 ) {}
   TSockPoint ( int af, int type, int protocol ) :
       TSocket ( af, type, protocol ), sourceEnabled ( 0 ) {}
   TSockPoint ( const SOCKET& sock ) :
       TSocket ( sock ), sourceEnabled ( 0 ) {}
  ~TSockPoint () {}

   void EnableSource ( int yes ) { sourceEnabled = yes ; }

   int  SetLocalName ( const TPointName& name ) ;
   int  ConnectTo    ( const TPointName& name ) ;

   void GetLocalName   ( TPointName& name ) const ;
   void GetConnectName ( TPointName& name ) const ;
} ;

class TSockDgram : public TSockPoint
{
  protected :
   TPointName defDest ; 

   int  DoSend ( int * size, const void * data, const TPointName * dest ) ;
   void Receive ( int, const void *, const TPointName * ) {} // write-only by default
  public:
   TSockDgram ( int af = AF_INET, int protocol = IPPROTO_UDP ) ;

   void SetDefaultDestination ( TPointName& name )   { defDest = name ; } 
} ; 



class TSockStream : public TSockPoint
{
   TPointName lastConnect ;
  protected :
   int connected ;
   void Process_FD_CONNECT ( int error );
   void Process_FD_CLOSE   ( int error );
   void TimerCallback () ;
   virtual void ConnectChanged () {}
  public:
   TSockStream () : connected ( 0 ) {}
   TSockStream ( int af, int protocol ) ;
   TSockStream ( const SOCKET& sock ) ;
  ~TSockStream () { Shutdown () ; }

   void SetSocket ( const SOCKET& sock ) ; 
   int  ConnectTo ( const TPointName& name ) ;
   int  IsConnected () const { return connected ; }
   int  Reconnect  () ;
   int  Shutdown () ;
} ;

// fragment stream treats first 4 bytes of incoming message as fragment length
class TFragmentStream : public TSockStream
{
  protected :
   void Receive ( int size, const void * data, const TPointName * from ) ;
  protected :
   char * buf ;     // partially read message buffer
   int    sz ;      // size of "buf"
   int    len ;     // number of bytes, read from current message 
   virtual void FragmentReceived ( int size, const void * data ) = 0 ;
  public :
   TFragmentStream () ;
   TFragmentStream ( int af, int protocol ) ;
   TFragmentStream ( const SOCKET& sock ) ;
  ~TFragmentStream () ;

   void SetSocket ( const SOCKET& sock ) ; 
   int  Reconnect  () ;
} ;



class TListener : public TSocket
{
  protected :
   void Process_FD_ACCEPT ( int error ) ;
   void SocketError ( int code ) const ;
   virtual void Accepted ( SOCKET sock ) = 0 ;
  public :
   TListener ( int af, int type, int protocol ) ;
  ~TListener () {}

   int  ListenAs       ( const TPointName& name ) ;
   void GetConnectName (       TPointName& name ) const ;
} ;


class TPinger : public TSocket
{
   struct IcmpHeader * m_pdata ; 
   ushort              m_echoid ; 
   ushort              m_seqno ; 
   BOOL                m_bpinging ; 
   BOOL                m_bNoReply ;  
   char                m_hostent[MAXGETHOSTSTRUCT] ; 
  protected :
   sockaddr_in         m_dest ; 
   char                m_host[100] ;

   void Process_FD_READ   ( int error ) ;
   void Process_SearchHost( int error ) ;   
   void SocketError       ( int code ) const {}
   void TimerCallback     () ;

  protected :
   virtual void EchoReceived ( BOOL * cancel ) = 0 ; 
   virtual void TimeOut      ()                = 0 ; 
  public :
   TPinger () ; 
  ~TPinger () ; 

   void Ping ( const char * hostname ) ; 
} ; 



#ifdef _MSC_VER 
  #pragma warning ( default : 4200)
  #pragma pack ( pop )
#endif
