#pragma once
#include "Interfaces.h"

struct FilesParser
{
   struct ItemEntry 
   {
      string name   ;
      string server ;
      string link   ; 
      bool   folder ;
      
      long   size   ;
      
      short  year   ;
      short  month  ;
      short  date   ;
      short  hour   ;
      short  minute ;
   };

   
   typedef std::vector< ItemEntry >    Items     ;
   typedef std::map < string, Items >  FolderList; // folder name & items in it 

   FilesParser( LPCSTR filename )  
   {
      FILE* f = fopen( filename, "r" ) ;
      if( f != NULL )
      {
         string      folder ;
         ItemEntry   item   ;

         while ( ParseFileEntry( f, folder, item ) != EOF )
            list_[ folder ].push_back( item ) ;            

         fclose( f ) ;
      }
   }

   FolderList const& GetFolderList() { return list_ ; }

private:
   int ParseFileEntry( FILE* descriptor, string& folder, ItemEntry& item )
   {
      USES_CONVERSION ;

      char folderName[ MAX_PATH ] ;
      char name      [ MAX_PATH ] ;
      char server    [ MAX_PATH ] ;
      char link      [ MAX_PATH ] ;

      char folderSign ;
      
      int res = fscanf( descriptor, "%s %s %s %s %c %d %d/%d/%d %d:%d", 
         folderName, name, server, link, &folderSign, &item.size, &item.year, &item.month,
         &item.date, &item.hour, &item.minute ) ; 

      item.name   = A2W( name   ) ;
      item.server = A2W( server ) ;
      item.link   = A2W( link   ) ;
      item.folder = folderSign == 'd' ;

      folder = A2W( folderName ) ;

      return res ;
   }

private:
   FolderList list_ ;
};

BOOL operator == ( FilesParser::ItemEntry const& it1, std::wstring const& str )
{
   return it1.name == str ;
}
